# -*- coding: utf-8 -*-
"""Parser for the Google Chrome extension activity database files.

The Chrome extension activity is stored in SQLite database files named
Extension Activity.
"""

from dfdatetime import webkit_time as dfdatetime_webkit_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class ChromeExtensionActivityEventData(events.EventData):
  """Chrome Extension Activity event data.

  Attributes:
    action_type (str): action type.
    activity_id (str): activity identifier.
    api_name (str): name of API.
    arg_url (str): URL argument.
    args (str): arguments.
    extension_id (str): extension identifier.
    other (str): other.
    page_title (str): title of webpage.
    page_url (str): URL of webpage.
  """

  DATA_TYPE = u'chrome:extension_activity:activity_log'

  def __init__(self):
    """Initializes event data."""
    super(ChromeExtensionActivityEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.action_type = None
    self.activity_id = None
    self.api_name = None
    self.arg_url = None
    self.args = None
    self.extension_id = None
    self.other = None
    self.page_title = None
    self.page_url = None


class ChromeExtensionActivityPlugin(interface.SQLitePlugin):
  """Plugin to parse Chrome extension activity database files."""

  NAME = u'chrome_extension_activity'
  DESCRIPTION = u'Parser for Chrome extension activity SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      ((u'SELECT time, extension_id, action_type, api_name, args, page_url, '
        u'page_title, arg_url, other, activity_id '
        u'FROM activitylog_uncompressed ORDER BY time'),
       u'ParseActivityLogUncompressedRow')]

  REQUIRED_TABLES = frozenset([
      u'activitylog_compressed', u'string_ids', u'url_ids'])

  SCHEMAS = [{
      u'activitylog_compressed': (
          u'CREATE TABLE activitylog_compressed (count INTEGER NOT NULL '
          u'DEFAULT 1, extension_id_x INTEGER NOT NULL, time INTEGER, '
          u'action_type INTEGER, api_name_x INTEGER, args_x INTEGER, '
          u'page_url_x INTEGER, page_title_x INTEGER, arg_url_x INTEGER, '
          u'other_x INTEGER)'),
      u'string_ids': (
          u'CREATE TABLE string_ids (id INTEGER PRIMARY KEY, value TEXT NOT '
          u'NULL)'),
      u'url_ids': (
          u'CREATE TABLE url_ids (id INTEGER PRIMARY KEY, value TEXT NOT '
          u'NULL)')}]

  def ParseActivityLogUncompressedRow(
      self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses an activity log row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    event_data = ChromeExtensionActivityEventData()
    event_data.action_type = row['action_type']
    event_data.activity_id = row['activity_id']
    event_data.api_name = row['api_name']
    event_data.arg_url = row['arg_url']
    event_data.args = row['args']
    event_data.extension_id = row['extension_id']
    event_data.other = row['other']
    event_data.page_title = row['page_title']
    event_data.page_url = row['page_url']
    event_data.query = query

    timestamp = row['time']
    date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_UNKNOWN)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(ChromeExtensionActivityPlugin)



# -*- coding: utf-8 -*-
"""Parser for the Google Chrome History files.

The Chrome History is stored in SQLite database files named History
and Archived History. Where the Archived History does not contain
the downloads table.
"""

from dfdatetime import posix_time as dfdatetime_posix_time
from dfdatetime import webkit_time as dfdatetime_webkit_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class ChromeHistoryFileDownloadedEventData(events.EventData):
  """Chrome History file downloaded event data.

  Attributes:
    full_path (str): full path where the file was downloaded to.
    received_bytes (int): number of bytes received while downloading.
    total_bytes (int): total number of bytes to download.
    url (str): URL of the downloaded file.
  """

  DATA_TYPE = u'chrome:history:file_downloaded'

  def __init__(self):
    """Initializes event data."""
    super(ChromeHistoryFileDownloadedEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.full_path = None
    self.received_bytes = None
    self.total_bytes = None
    self.url = None


class ChromeHistoryPageVisitedEventData(events.EventData):
  """Chrome History page visited event data.

  Attributes:
    extra (str): extra event data.
    from_visit (str): URL where the visit originated from.
    hostname (str): visited hostname.
    title (str): title of the visited page.
    typed_count (int): number of characters of the URL that were typed.
    url (str): URL of the visited page.
    visit_source (str): source of the page visit.
  """

  DATA_TYPE = u'chrome:history:page_visited'

  def __init__(self):
    """Initializes event data."""
    super(ChromeHistoryPageVisitedEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.extra = None
    self.from_visit = None
    self.host = None
    self.title = None
    self.typed_count = None
    self.url = None
    self.visit_source = None


class ChromeHistoryPlugin(interface.SQLitePlugin):
  """Parse Chrome Archived History and History files."""

  NAME = u'chrome_history'
  DESCRIPTION = u'Parser for Chrome history SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      ((u'SELECT urls.id, urls.url, urls.title, urls.visit_count, '
        u'urls.typed_count, urls.last_visit_time, urls.hidden, visits.'
        u'visit_time, visits.from_visit, visits.transition, visits.id '
        u'AS visit_id FROM urls, visits WHERE urls.id = visits.url ORDER '
        u'BY visits.visit_time'), u'ParseLastVisitedRow'),
      ((u'SELECT downloads.id AS id, downloads.start_time,'
        u'downloads.target_path, downloads_url_chains.url, '
        u'downloads.received_bytes, downloads.total_bytes FROM downloads,'
        u' downloads_url_chains WHERE downloads.id = '
        u'downloads_url_chains.id'), u'ParseNewFileDownloadedRow'),
      ((u'SELECT id, full_path, url, start_time, received_bytes, '
        u'total_bytes,state FROM downloads'), u'ParseFileDownloadedRow')]

  # The required tables common to Archived History and History.
  REQUIRED_TABLES = frozenset([
      u'keyword_search_terms', u'meta', u'urls', u'visits', u'visit_source'])

  SCHEMAS = [{
      u'downloads': (
          u'CREATE TABLE downloads (id INTEGER PRIMARY KEY, full_path '
          u'LONGVARCHAR NOT NULL, url LONGVARCHAR NOT NULL, start_time INTEGER '
          u'NOT NULL, received_bytes INTEGER NOT NULL, total_bytes INTEGER NOT '
          u'NULL, state INTEGER NOT NULL)'),
      u'keyword_search_terms': (
          u'CREATE TABLE keyword_search_terms (keyword_id INTEGER NOT NULL, '
          u'url_id INTEGER NOT NULL, lower_term LONGVARCHAR NOT NULL, term '
          u'LONGVARCHAR NOT NULL)'),
      u'meta': (
          u'CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY, '
          u'value LONGVARCHAR)'),
      u'presentation': (
          u'CREATE TABLE presentation(url_id INTEGER PRIMARY KEY, pres_index '
          u'INTEGER NOT NULL)'),
      u'segment_usage': (
          u'CREATE TABLE segment_usage (id INTEGER PRIMARY KEY, segment_id '
          u'INTEGER NOT NULL, time_slot INTEGER NOT NULL, visit_count INTEGER '
          u'DEFAULT 0 NOT NULL)'),
      u'segments': (
          u'CREATE TABLE segments (id INTEGER PRIMARY KEY, name VARCHAR, '
          u'url_id INTEGER NON NULL, pres_index INTEGER DEFAULT -1 NOT NULL)'),
      u'urls': (
          u'CREATE TABLE urls(id INTEGER PRIMARY KEY, url LONGVARCHAR, title '
          u'LONGVARCHAR, visit_count INTEGER DEFAULT 0 NOT NULL, typed_count '
          u'INTEGER DEFAULT 0 NOT NULL, last_visit_time INTEGER NOT NULL, '
          u'hidden INTEGER DEFAULT 0 NOT NULL, favicon_id INTEGER DEFAULT 0 '
          u'NOT NULL)'),
      u'visit_source': (
          u'CREATE TABLE visit_source(id INTEGER PRIMARY KEY, source INTEGER '
          u'NOT NULL)'),
      u'visits': (
          u'CREATE TABLE visits(id INTEGER PRIMARY KEY, url INTEGER NOT NULL, '
          u'visit_time INTEGER NOT NULL, from_visit INTEGER, transition '
          u'INTEGER DEFAULT 0 NOT NULL, segment_id INTEGER, is_indexed '
          u'BOOLEAN)')}]

  # Queries for cache building.
  URL_CACHE_QUERY = (
      u'SELECT visits.id AS id, urls.url, urls.title FROM '
      u'visits, urls WHERE urls.id = visits.url')
  SYNC_CACHE_QUERY = u'SELECT id, source FROM visit_source'

  # The following definition for values can be found here:
  # http://src.chromium.org/svn/trunk/src/content/public/common/ \
  # page_transition_types_list.h
  PAGE_TRANSITION = {
      0: u'LINK',
      1: u'TYPED',
      2: u'AUTO_BOOKMARK',
      3: u'AUTO_SUBFRAME',
      4: u'MANUAL_SUBFRAME',
      5: u'GENERATED',
      6: u'START_PAGE',
      7: u'FORM_SUBMIT',
      8: u'RELOAD',
      9: u'KEYWORD',
      10: u'KEYWORD_GENERATED '
  }

  TRANSITION_LONGER = {
      0: u'User clicked a link',
      1: u'User typed the URL in the URL bar',
      2: u'Got through a suggestion in the UI',
      3: (u'Content automatically loaded in a non-toplevel frame - user may '
          u'not realize'),
      4: u'Subframe explicitly requested by the user',
      5: (u'User typed in the URL bar and selected an entry from the list - '
          u'such as a search bar'),
      6: u'The start page of the browser',
      7: u'A form the user has submitted values to',
      8: (u'The user reloaded the page, eg by hitting the reload button or '
          u'restored a session'),
      9: (u'URL what was generated from a replaceable keyword other than the '
          u'default search provider'),
      10: u'Corresponds to a visit generated from a KEYWORD'
  }

  # The following is the values for the source enum found in the visit_source
  # table and describes where a record originated from (if it originates from a
  # different storage than locally generated).
  # The source can be found here:
  # http://src.chromium.org/svn/trunk/src/chrome/browser/history/\
  # history_types.h
  VISIT_SOURCE = {
      0: u'SOURCE_SYNCED',
      1: u'SOURCE_BROWSED',
      2: u'SOURCE_EXTENSION',
      3: u'SOURCE_FIREFOX_IMPORTED',
      4: u'SOURCE_IE_IMPORTED',
      5: u'SOURCE_SAFARI_IMPORTED'
  }

  CORE_MASK = 0xff

  def _GetHostname(self, url):
    """Retrieves the hostname from a full URL.

    Args:
      url (str): full URL.

    Returns:
      str: hostname or full URL if not hostname could be retrieved.
    """
    if url.startswith(u'http') or url.startswith(u'ftp'):
      _, _, uri = url.partition(u'//')
      hostname, _, _ = uri.partition(u'/')
      return hostname

    if url.startswith(u'about') or url.startswith(u'chrome'):
      hostname, _, _ = url.partition(u'/')
      return hostname

    return url

  def _GetUrl(self, url, cache, database):
    """Retrieves an URL from a reference to an entry in the from_visit table.

    Args:
      url (str): URL.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.
    """
    if not url:
      return u''

    url_cache_results = cache.GetResults(u'url')
    if not url_cache_results:
      result_set = database.Query(self.URL_CACHE_QUERY)

      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      cache.CacheQueryResults(result_set, 'url', 'id', ('url', 'title'))
      url_cache_results = cache.GetResults(u'url')

    reference_url, reference_title = url_cache_results.get(url, [u'', u''])

    if not reference_url:
      return u''

    return u'{0:s} ({1:s})'.format(reference_url, reference_title)

  def _GetVisitSource(self, visit_id, cache, database):
    """Return a string denoting the visit source type if possible.

    Args:
      visit_id (str): ID from the visits table for the particular record.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.

    Returns:
      str: visit source or None if not found.
    """
    if not visit_id:
      return

    sync_cache_results = cache.GetResults(u'sync')
    if not sync_cache_results:
      result_set = database.Query(self.SYNC_CACHE_QUERY)

      # Note that pysqlite does not accept a Unicode string in row['string'] and
      # will raise "IndexError: Index must be int or string".
      cache.CacheQueryResults(result_set, 'sync', 'id', ('source',))
      sync_cache_results = cache.GetResults(u'sync')

    results = sync_cache_results.get(visit_id, None)
    if results is None:
      return

    return self.VISIT_SOURCE.get(results, None)

  def ParseFileDownloadedRow(
      self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a file downloaded row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    event_data = ChromeHistoryFileDownloadedEventData()
    event_data.full_path = row['full_path']
    event_data.offset = row['id']
    event_data.query = query
    event_data.received_bytes = row['received_bytes']
    event_data.total_bytes = row['total_bytes']
    event_data.url = row['url']

    timestamp = row['start_time']
    date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_FILE_DOWNLOADED)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseNewFileDownloadedRow(
      self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a file downloaded row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    event_data = ChromeHistoryFileDownloadedEventData()
    event_data.full_path = row['target_path']
    event_data.offset = row['id']
    event_data.query = query
    event_data.received_bytes = row['received_bytes']
    event_data.total_bytes = row['total_bytes']
    event_data.url = row['url']

    timestamp = row['start_time']
    date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_FILE_DOWNLOADED)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseLastVisitedRow(
      self, parser_mediator, row, cache=None, database=None, query=None,
      **unused_kwargs):
    """Parses a last visited row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      cache (Optional[SQLiteCache]): cache.
      database (Optional[SQLiteDatabase]): database.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    extras = []

    transition_nr = row['transition'] & self.CORE_MASK
    page_transition = self.PAGE_TRANSITION.get(transition_nr, u'')
    if page_transition:
      extras.append(u'Type: [{0:s} - {1:s}]'.format(
          page_transition, self.TRANSITION_LONGER.get(transition_nr, u'')))

    if row['hidden'] == u'1':
      extras.append(u'(url hidden)')

    # TODO: move to formatter.
    count = row['typed_count']
    if count >= 1:
      if count > 1:
        multi = u's'
      else:
        multi = u''

      extras.append(u'(type count {1:d} time{0:s})'.format(multi, count))
    else:
      extras.append(u'(URL not typed directly - no typed count)')

    visit_source = self._GetVisitSource(row['visit_id'], cache, database)

    # TODO: replace extras by conditional formatting.
    event_data = ChromeHistoryPageVisitedEventData()
    event_data.extra = u' '.join(extras)
    event_data.from_visit = self._GetUrl(row['from_visit'], cache, database)
    event_data.host = self._GetHostname(row['url'])
    event_data.offset = row['id']
    event_data.query = query
    event_data.title = row['title']
    event_data.typed_count = row['typed_count']
    event_data.url = row['url']
    event_data.visit_source = visit_source

    timestamp = row['visit_time']
    date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_LAST_VISITED)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(ChromeHistoryPlugin)



# -*- coding: utf-8 -*-
"""This file contains preprocessors for MacOS."""

import abc
import plistlib

from plaso.containers import artifacts
from plaso.lib import errors
from plaso.lib import plist
from plaso.parsers.plist_plugins import interface as plist_interface
from plaso.preprocessors import interface
from plaso.preprocessors import manager


class PlistFileArtifactPreprocessorPlugin(
    interface.FileArtifactPreprocessorPlugin):
  """Plist file artifact preprocessor plugin interface.

  Retrieves values from a plist file artifact using names of keys defined
  in _PLIST_KEYS.
  """

  # The key that's value should be returned back. It is an ordered list
  # of preference. If the first value is found it will be returned and no
  # others will be searched.
  _PLIST_KEYS = [u'']

  def _FindKeys(self, key, names, matches):
    """Searches the plist key hierarchy for keys with matching names.

    If a match is found a tuple of the key name and value is added to
    the matches list.

    Args:
      key (plistlib._InternalDict): plist key.
      names (list[str]): names of the keys to match.
      matches (list[str]): keys with matching names.
    """
    for name, subkey in iter(key.items()):
      if name in names:
        matches.append((name, subkey))

      # pylint: disable=protected-access
      if isinstance(subkey, plistlib._InternalDict):
        self._FindKeys(subkey, names, matches)

  def _ParseFileData(self, knowledge_base, file_object):
    """Parses file content (data) for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_object (dfvfs.FileIO): file-like object that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    plist_file = plist.PlistFile()

    try:
      plist_file.Read(file_object)

    except IOError as exception:
      raise errors.PreProcessFail(
          u'Unable to read: {0:s} with error: {1!s}'.format(
              self.ARTIFACT_DEFINITION_NAME, exception))

    if not plist_file.root_key:
      raise errors.PreProcessFail((
          u'Unable to read: {0:s} with error: missing root key').format(
              self.ARTIFACT_DEFINITION_NAME))

    matches = []

    self._FindKeys(plist_file.root_key, self._PLIST_KEYS, matches)
    if not matches:
      raise errors.PreProcessFail(
          u'Unable to read: {0:s} with error: no such keys: {1:s}.'.format(
              self.ARTIFACT_DEFINITION_NAME, u', '.join(self._PLIST_KEYS)))

    name = None
    value = None
    for name, value in matches:
      if value:
        break

    if value is None:
      raise errors.PreProcessFail((
          u'Unable to read: {0:s} with error: no values found for keys: '
          u'{1:s}.').format(
              self.ARTIFACT_DEFINITION_NAME, u', '.join(self._PLIST_KEYS)))

    return self._ParsePlistKeyValue(knowledge_base, name, value)

  @abc.abstractmethod
  def _ParsePlistKeyValue(self, knowledge_base, name, value):
    """Parses a plist key value.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      name (str): name of the plist key.
      value (str): value of the plist key.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.
    """


class MacOSHostnamePlugin(PlistFileArtifactPreprocessorPlugin):
  """MacOS hostname plugin."""

  ARTIFACT_DEFINITION_NAME = u'MacOSSystemConfigurationPreferencesPlistFile'

  _PLIST_KEYS = [u'ComputerName', u'LocalHostName']

  def _ParsePlistKeyValue(self, knowledge_base, name, value):
    """Parses a plist key value.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      name (str): name of the plist key.
      value (str): value of the plist key.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.
    """
    if name in self._PLIST_KEYS:
      hostname_artifact = artifacts.HostnameArtifact(name=value)
      knowledge_base.SetHostname(hostname_artifact)

    return name in self._PLIST_KEYS


class MacOSKeyboardLayoutPlugin(PlistFileArtifactPreprocessorPlugin):
  """MacOS keyboard layout plugin."""

  ARTIFACT_DEFINITION_NAME = u'MacOSKeyboardLayoutPlistFile'

  _PLIST_KEYS = [u'AppleCurrentKeyboardLayoutInputSourceID']

  def _ParsePlistKeyValue(self, knowledge_base, name, value):
    """Parses a plist key value.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      name (str): name of the plist key.
      value (str): value of the plist key.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.
    """
    if name in self._PLIST_KEYS:
      if isinstance(value, (list, tuple)):
        value = value[0]

      _, _, keyboard_layout = value.rpartition(u'.')

      knowledge_base.SetValue(u'keyboard_layout', keyboard_layout)

    return name in self._PLIST_KEYS


class MacOSSystemVersionPlugin(PlistFileArtifactPreprocessorPlugin):
  """MacOS system version information plugin."""

  ARTIFACT_DEFINITION_NAME = u'MacOSSystemVersionPlistFile'

  _PLIST_KEYS = [u'ProductUserVisibleVersion']

  def _ParsePlistKeyValue(self, knowledge_base, name, value):
    """Parses a plist key value.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      name (str): name of the plist key.
      value (str): value of the plist key.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.
    """
    if name in self._PLIST_KEYS:
      knowledge_base.SetValue(u'operating_system_version', value)

    return name in self._PLIST_KEYS


class MacOSTimeZonePlugin(interface.FileEntryArtifactPreprocessorPlugin):
  """MacOS time zone plugin."""

  ARTIFACT_DEFINITION_NAME = u'MacOSLocalTime'

  def _ParseFileEntry(self, knowledge_base, file_entry):
    """Parses artifact file system data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_entry (dfvfs.FileEntry): file entry that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not file_entry or not file_entry.link:
      raise errors.PreProcessFail(
          u'Unable to read: {0:s} with error: not a symbolic link'.format(
              self.ARTIFACT_DEFINITION_NAME))

    result = False

    _, _, time_zone = file_entry.link.partition(u'zoneinfo/')
    if time_zone:
      try:
        knowledge_base.SetTimeZone(time_zone)
        result = True
      except ValueError:
        # TODO: add and store preprocessing errors.
        pass

    return result


class MacOSUserAccountsPlugin(interface.FileEntryArtifactPreprocessorPlugin):
  """MacOS user accounts plugin."""

  ARTIFACT_DEFINITION_NAME = u'MacOSUserPasswordHashesPlistFiles'

  _KEYS = frozenset([u'gid', 'home', u'name', u'realname', u'shell', u'uid'])

  def _GetKeysDefaultEmpty(self, top_level, keys, depth=1):
    """Retrieves plist keys, defaulting to empty values.

    Args:
      top_level (plistlib._InternalDict): top level plist object.
      keys (set[str]): names of keys that should be returned.
      depth (int): depth within the plist, where 1 is top level.

    Returns:
      dict[str, str]: values of the requested keys.
    """
    keys = set(keys)
    match = {}

    if depth == 1:
      for key in keys:
        value = top_level.get(key, None)
        if value is not None:
          match[key] = value
    else:
      for _, parsed_key, parsed_value in plist_interface.RecurseKey(
          top_level, depth=depth):
        if parsed_key in keys:
          match[parsed_key] = parsed_value
          if set(match.keys()) == keys:
            return match
    return match

  def _GetPlistRootKey(self, file_entry):
    """Retrieves the root key of a plist file.

    Args:
      file_entry (dfvfs.FileEntry): file entry of the plist.

    Returns:
      plistlib._InternalDict: plist root key.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    file_object = file_entry.GetFileObject()

    try:
      plist_file = plist.PlistFile()
      plist_file.Read(file_object)

    except IOError as exception:
      location = getattr(file_entry.path_spec, u'location', u'')
      raise errors.PreProcessFail(
          u'Unable to read plist file: {0:s} with error: {1:s}'.format(
              location, exception))

    finally:
      file_object.close()

    return plist_file.root_key

  def _ParseFileEntry(self, knowledge_base, file_entry):
    """Parses artifact file system data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_entry (dfvfs.FileEntry): file entry that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    root_key = self._GetPlistRootKey(file_entry)
    if not root_key:
      location = getattr(file_entry.path_spec, u'location', u'')
      raise errors.PreProcessFail((
          u'Unable to read: {0:s} plist: {1:s} with error: missing root '
          u'key.').format(self.ARTIFACT_DEFINITION_NAME, location))

    try:
      match = self._GetKeysDefaultEmpty(root_key, self._KEYS)
    except KeyError as exception:
      location = getattr(file_entry.path_spec, u'location', u'')
      raise errors.PreProcessFail(
          u'Unable to read: {0:s} plist: {1:s} with error: {2!s}'.format(
              self.ARTIFACT_DEFINITION_NAME, location, exception))

    name = match.get(u'name', [None])[0]
    uid = match.get(u'uid', [None])[0]

    if not name or not uid:
      # TODO: add and store preprocessing errors.
      return False

    user_account = artifacts.UserAccountArtifact(
        identifier=uid, username=name)
    user_account.group_identifier = match.get(u'gid', [None])[0]
    user_account.full_name = match.get(u'realname', [None])[0]
    user_account.shell = match.get(u'shell', [None])[0]
    user_account.user_directory = match.get(u'home', [None])[0]

    try:
      knowledge_base.AddUserAccount(user_account)
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass

    return False


manager.PreprocessPluginsManager.RegisterPlugins([
    MacOSHostnamePlugin, MacOSKeyboardLayoutPlugin, MacOSSystemVersionPlugin,
    MacOSTimeZonePlugin, MacOSUserAccountsPlugin])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests the single process processing engine."""

import unittest

from artifacts import reader as artifacts_reader
from artifacts import registry as artifacts_registry
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import context

from plaso.containers import sessions
from plaso.engine import configurations
from plaso.engine import single_process
from plaso.storage import fake_storage

from tests import test_lib as shared_test_lib


class SingleProcessEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the single process engine object."""

  # pylint: disable=protected-access

  @shared_test_lib.skipUnlessHasTestFile([u'artifacts'])
  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testProcessSources(self):
    """Tests the ProcessSources function."""
    registry = artifacts_registry.ArtifactDefinitionsRegistry()
    reader = artifacts_reader.YamlArtifactsReader()
    path = shared_test_lib.GetTestFilePath([u'artifacts'])
    registry.ReadFromDirectory(reader, path)

    test_engine = single_process.SingleProcessEngine()
    resolver_context = context.Context()
    session = sessions.Session()

    source_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=source_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=os_path_spec)

    test_engine.PreprocessSources(registry, [source_path_spec])

    storage_writer = fake_storage.FakeStorageWriter(session)

    configuration = configurations.ProcessingConfiguration()
    configuration.parser_filter_expression = u'filestat'

    test_engine.ProcessSources(
        [source_path_spec], storage_writer, resolver_context, configuration)

    self.assertEqual(storage_writer.number_of_events, 15)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the cookie plugins manager."""

import unittest

from plaso.parsers.cookie_plugins import interface
from plaso.parsers.cookie_plugins import manager


class TestCookiePlugin(interface.BaseCookiePlugin):
  """Test cookie plugin."""

  NAME = u'test_cookie_plugin'
  DESCRIPTION = u'Test cookie plugin.'

  def GetEntries(
      self, unused_parser_mediator, cookie_data=None, url=None, **kwargs):
    """Extract and return EventObjects from the data structure.

    Args:
      parser_mediator: A parser mediator object (instance of ParserMediator).
      cookie_data: Optional cookie data, as a byte string.
      url: Optional URL or path where the cookie got set.
    """
    return


class CookiePluginsManagerTest(unittest.TestCase):
  """Tests for the cookie plugins manager."""

  def testPluginRegistration(self):
    """Tests the RegisterPlugin and DeregisterPlugin functions."""
    # pylint: disable=protected-access
    number_of_plugins = len(manager.CookiePluginsManager._plugin_classes)

    manager.CookiePluginsManager.RegisterPlugin(TestCookiePlugin)
    self.assertEqual(
        len(manager.CookiePluginsManager._plugin_classes),
        number_of_plugins + 1)

    with self.assertRaises(KeyError):
      manager.CookiePluginsManager.RegisterPlugin(TestCookiePlugin)

    manager.CookiePluginsManager.DeregisterPlugin(TestCookiePlugin)
    self.assertEqual(
        len(manager.CookiePluginsManager._plugin_classes),
        number_of_plugins)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the util functions."""

import logging
import os
import tempfile
import unittest

from dfvfs.helpers import file_system_searcher
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import artifacts
from plaso.frontend import utils

from tests import test_lib as shared_test_lib


class BuildFindSpecsFromFileTest(shared_test_lib.BaseTestCase):
  """Tests for the BuildFindSpecsFromFile function."""

  @shared_test_lib.skipUnlessHasTestFile([u'System.evtx'])
  @shared_test_lib.skipUnlessHasTestFile([u'testdir', u'filter_1.txt'])
  @shared_test_lib.skipUnlessHasTestFile([u'testdir', u'filter_3.txt'])
  def testBuildFindSpecsFromFile(self):
    """Tests the BuildFindSpecsFromFile function."""
    filter_file_path = u''
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
      filter_file_path = temp_file.name
      # 2 hits.
      temp_file.write(b'/test_data/testdir/filter_.+.txt\n')
      # A single hit.
      temp_file.write(b'/test_data/.+evtx\n')
      # A single hit.
      temp_file.write(b'/AUTHORS\n')
      temp_file.write(b'/does_not_exist/some_file_[0-9]+txt\n')
      # Path expansion.
      temp_file.write(b'{systemroot}/Tasks/.+[.]job\n')
      # This should not compile properly, missing file information.
      temp_file.write(b'failing/\n')
      # This should not fail during initial loading, but fail later on.
      temp_file.write(b'bad re (no close on that parenthesis/file\n')

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=u'SystemRoot', value=u'C:\\Windows')

    find_specs = utils.BuildFindSpecsFromFile(
        filter_file_path, environment_variables=[environment_variable])

    try:
      os.remove(filter_file_path)
    except (OSError, IOError) as exception:
      logging.warning(
          u'Unable to remove filter file: {0:s} with error: {1:s}'.format(
              filter_file_path, exception))

    self.assertEqual(len(find_specs), 5)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=u'.')
    file_system = path_spec_resolver.Resolver.OpenFileSystem(path_spec)
    searcher = file_system_searcher.FileSystemSearcher(
        file_system, path_spec)

    path_spec_generator = searcher.Find(find_specs=find_specs)
    self.assertIsNotNone(path_spec_generator)

    path_specs = list(path_spec_generator)
    # Two evtx, one AUTHORS, two filter_*.txt files, total 5 files.
    self.assertEqual(len(path_specs), 5)

    with self.assertRaises(IOError):
      utils.BuildFindSpecsFromFile('thisfiledoesnotexist')

    file_system.Close()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the attribute container interface."""

import unittest

from plaso.containers import interface

from tests import test_lib as shared_test_lib


class AttributeContainerIdentifierTest(shared_test_lib.BaseTestCase):
  """Tests for the attribute container identifier."""

  def testCopyToString(self):
    """Tests the CopyToString function."""
    identifier = interface.AttributeContainerIdentifier()

    expected_identifier_string = u'{0:d}'.format(id(identifier))
    identifier_string = identifier.CopyToString()
    self.assertEqual(identifier_string, expected_identifier_string)


class AttributeContainerTest(shared_test_lib.BaseTestCase):
  """Tests for the attribute container interface."""

  def testCopyToDict(self):
    """Tests the CopyToDict function."""
    attribute_container = interface.AttributeContainer()
    attribute_container.attribute_name = u'attribute_name'
    attribute_container.attribute_value = u'attribute_value'

    expected_dict = {
        u'attribute_name': u'attribute_name',
        u'attribute_value': u'attribute_value'}

    test_dict = attribute_container.CopyToDict()

    self.assertEqual(test_dict, expected_dict)

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = interface.AttributeContainer()
    attribute_container.attribute_name = u'attribute_name'
    attribute_container.attribute_value = u'attribute_value'

    expected_attribute_names = [u'attribute_name', u'attribute_value']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)

  def testGetAttributes(self):
    """Tests the GetAttributes function."""
    attribute_container = interface.AttributeContainer()
    attribute_container.attribute_name = u'attribute_name'
    attribute_container.attribute_value = u'attribute_value'

    expected_attributes = [
        (u'attribute_name', u'attribute_name'),
        (u'attribute_value', u'attribute_value')]

    attributes = sorted(attribute_container.GetAttributes())

    self.assertEqual(attributes, expected_attributes)

  def testGetAttributeValuesString(self):
    """Tests the GetAttributeValuesString function."""
    attribute_container = interface.AttributeContainer()
    attribute_container.attribute_name = u'attribute_name'
    attribute_container.attribute_value = u'attribute_value'

    attribute_values_string1 = attribute_container.GetAttributeValuesString()

    attribute_container.attribute_value = u'changes'

    attribute_values_string2 = attribute_container.GetAttributeValuesString()

    self.assertNotEqual(attribute_values_string1, attribute_values_string2)

  def testGetIdentifier(self):
    """Tests the GetIdentifier function."""
    attribute_container = interface.AttributeContainer()

    identifier = attribute_container.GetIdentifier()

    self.assertIsNotNone(identifier)

  def testGetSessionIdentifier(self):
    """Tests the GetSessionIdentifier function."""
    attribute_container = interface.AttributeContainer()

    session_identifier = attribute_container.GetSessionIdentifier()

    self.assertIsNone(session_identifier)

  def testSetIdentifier(self):
    """Tests the SetIdentifier function."""
    attribute_container = interface.AttributeContainer()

    attribute_container.SetIdentifier(None)

  def testSetSessionIdentifier(self):
    """Tests the SetSessionIdentifier function."""
    attribute_container = interface.AttributeContainer()

    attribute_container.SetSessionIdentifier(None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""gzip-based storage.

Only supports task storage at the moment.
"""

import gzip
import os
import time

from plaso.lib import definitions
from plaso.lib import platform_specific
from plaso.storage import event_heaps
from plaso.storage import identifiers
from plaso.storage import interface


class GZIPStorageFile(interface.BaseFileStorage):
  """gzip-based storage file."""

  # pylint: disable=abstract-method

  _COMPRESSION_LEVEL = 9

  _DATA_BUFFER_SIZE = 1 * 1024 * 1024

  def __init__(self, storage_type=definitions.STORAGE_TYPE_TASK):
    """Initializes a storage.

    Args:
      storage_type (Optional[str]): storage type.

    Raises:
      ValueError: if the storage type is not supported.
    """
    if storage_type != definitions.STORAGE_TYPE_TASK:
      raise ValueError(u'Unsupported storage type: {0:s}.'.format(
          storage_type))

    super(GZIPStorageFile, self).__init__()
    self._attribute_containers = {}
    self._gzip_file = None
    self._number_of_containers = 0

  def _AddAttributeContainer(self, attribute_container):
    """Adds an attribute container.

    Args:
      attribute_container (AttributeContainer): attribute container.
    """
    container_list = self._GetAttributeContainerList(
        attribute_container.CONTAINER_TYPE)

    container_list.append(attribute_container)

  def _GetAttributeContainerList(self, container_type):
    """Retrieves an attribute container list.

    Args:
      container_type (str): attribute container type.

    Returns:
      list[AttributeContainer]: attribute container list.
    """
    if container_type not in self._attribute_containers:
      self._attribute_containers[container_type] = []

    return self._attribute_containers[container_type]

  def _OpenRead(self):
    """Opens the storage file for reading."""
    # Do not use gzip.readlines() here since it can consume a large amount
    # of memory.
    data_buffer = self._gzip_file.read(self._DATA_BUFFER_SIZE)
    while data_buffer:
      lines = data_buffer.splitlines(True)
      data_buffer = b''
      for index, line in enumerate(lines):
        if line.endswith(b'\n'):
          attribute_container = self._DeserializeAttributeContainer(
              u'attribute_container', line)
          self._AddAttributeContainer(attribute_container)
        else:
          data_buffer = b''.join(lines[index:])
      data_buffer = data_buffer + self._gzip_file.read(
          self._DATA_BUFFER_SIZE)

  def _WriteAttributeContainer(self, attribute_container):
    """Writes an attribute container.

    Args:
      attribute_container (AttributeContainer): attribute container.
    """
    identifier = identifiers.SerializedStreamIdentifier(
        1, self._number_of_containers)
    attribute_container.SetIdentifier(identifier)

    serialized_data = self._SerializeAttributeContainer(attribute_container)

    self._number_of_containers += 1

    self._gzip_file.write(serialized_data)
    self._gzip_file.write(b'\n')

  def AddAnalysisReport(self, analysis_report):
    """Adds an analysis report.

    Args:
      analysis_report (AnalysisReport): analysis report.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(analysis_report)

  def AddError(self, error):
    """Adds an error.

    Args:
      error (ExtractionError): error.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(error)

  def AddEvent(self, event):
    """Adds an event.

    Args:
      event (EventObject): event.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    # TODO: change to no longer allow event_data_identifier is None
    # after refactoring every parser to generate event data.
    event_data_identifier = event.GetEventDataIdentifier()
    if event_data_identifier:
      if not isinstance(
          event_data_identifier, identifiers.SerializedStreamIdentifier):
        raise IOError(u'Unsupported event data identifier type: {0:s}'.format(
            type(event_data_identifier)))

      event.event_data_stream_number = event_data_identifier.stream_number
      event.event_data_entry_index = event_data_identifier.entry_index

    self._WriteAttributeContainer(event)

  def AddEventData(self, event_data):
    """Adds event data.

    Args:
      event_data (EventData): event data.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(event_data)

  def AddEventSource(self, event_source):
    """Adds an event source.

    Args:
      event_source (EventSource): event source.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(event_source)

  def AddEventTag(self, event_tag):
    """Adds an event tag.

    Args:
      event_tag (EventTag): event tag.

    Raises:
      IOError: when the storage file is closed or read-only or
          if the event tag event identifier type is not supported.
    """
    self._RaiseIfNotWritable()

    event_identifier = event_tag.GetEventIdentifier()
    if not isinstance(
        event_identifier, identifiers.SerializedStreamIdentifier):
      raise IOError(u'Unsupported event identifier type: {0:s}'.format(
          type(event_identifier)))

    event_tag.event_stream_number = event_identifier.stream_number
    event_tag.event_entry_index = event_identifier.entry_index

    self._WriteAttributeContainer(event_tag)

  def Close(self):
    """Closes the storage.

    Raises:
      IOError: if the storage file is already closed.
    """
    if not self._is_open:
      raise IOError(u'Storage file already closed.')

    if self._gzip_file:
      self._gzip_file.close()
      self._gzip_file = None
    self._is_open = False

  def GetAnalysisReports(self):
    """Retrieves the analysis reports.

    Returns:
      generator(AnalysisReport): analysis report generator.
    """
    return iter(self._GetAttributeContainerList(u'analysis_report'))

  def GetErrors(self):
    """Retrieves the errors.

    Returns:
      generator(ExtractionError): error generator.
    """
    return iter(self._GetAttributeContainerList(u'extraction_error'))

  def GetEventData(self):
    """Retrieves event data.

    Returns:
      generator(EventData): event data generator.
    """
    return iter(self._GetAttributeContainerList(u'event_data'))

  def GetEventDataByIdentifier(self, identifier):
    """Retrieves specific event data.

    Args:
      identifier (AttributeContainerIdentifier): event data identifier.

    Returns:
      EventData: event data or None if not available.
    """
    raise NotImplementedError()

  def GetEvents(self):
    """Retrieves the events.

    Returns:
      generator(EventObject): event generator.
    """
    return iter(self._GetAttributeContainerList(u'event'))

  def GetEventSources(self):
    """Retrieves the event sources.

    Returns:
      generator(EventSource): event source generator.
    """
    return iter(self._GetAttributeContainerList(u'event_source'))

  def GetEventTags(self):
    """Retrieves the event tags.

    Yields:
      EventTag: event tag.
    """
    for event_tag in iter(self._GetAttributeContainerList(u'event_tag')):
      event_identifier = identifiers.SerializedStreamIdentifier(
          event_tag.event_stream_number, event_tag.event_entry_index)
      event_tag.SetEventIdentifier(event_identifier)

      yield event_tag

  def GetSortedEvents(self, time_range=None):
    """Retrieves the events in increasing chronological order.

    Args:
      time_range (Optional[TimeRange]): time range used to filter events
          that fall in a specific period.

    Returns:
      generator(EventObject): event generator.
    """
    event_heap = event_heaps.EventHeap()

    for event in self._GetAttributeContainerList(u'event'):
      if (time_range and (
          event.timestamp < time_range.start_timestamp or
          event.timestamp > time_range.end_timestamp)):
        continue

      event_heap.PushEvent(event)

    return iter(event_heap.PopEvents())

  def HasAnalysisReports(self):
    """Determines if a storage contains analysis reports.

    Returns:
      bool: True if the storage contains analysis reports.
    """
    return len(self._GetAttributeContainerList(u'analysis_report')) > 0

  def HasErrors(self):
    """Determines if a storage contains extraction errors.

    Returns:
      bool: True if the storage contains extraction errors.
    """
    return len(self._GetAttributeContainerList(u'extraction_error')) > 0

  def HasEventTags(self):
    """Determines if a storage contains event tags.

    Returns:
      bool: True if the storage contains event tags.
    """
    return len(self._GetAttributeContainerList(u'event_tags')) > 0

  # pylint: disable=arguments-differ
  def Open(self, path=None, read_only=True, **unused_kwargs):
    """Opens the storage.

    Args:
      path (Optional[str]): path of the storage file.
      read_only (Optional[bool]): True if the file should be opened in
          read-only mode.

    Raises:
      IOError: if the storage file is already opened.
      ValueError: if path is missing.
    """
    if self._is_open:
      raise IOError(u'Storage file already opened.')

    if not path:
      raise ValueError(u'Missing path.')

    if read_only:
      access_mode = 'rb'
    else:
      access_mode = 'wb'

    self._gzip_file = gzip.open(path, access_mode, self._COMPRESSION_LEVEL)
    if platform_specific.PlatformIsWindows():
      file_handle = self._gzip_file.fileno()
      platform_specific.DisableWindowsFileHandleInheritance(file_handle)

    if read_only:
      self._OpenRead()

    self._is_open = True
    self._read_only = read_only

  def WriteSessionCompletion(self, session_completion):
    """Writes session completion information.

    Args:
      session_completion (SessionCompletion): session completion information.
    """
    raise NotImplementedError()

  def WriteSessionStart(self, session_start):
    """Writes session start information.

    Args:
      session_start (SessionStart): session start information.
    """
    raise NotImplementedError()

  def WriteTaskCompletion(self, task_completion):
    """Writes task completion information.

    Args:
      task_completion (TaskCompletion): task completion information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(task_completion)

  def WriteTaskStart(self, task_start):
    """Writes task start information.

    Args:
      task_start (TaskStart): task start information.

    Raises:
      IOError: when the storage file is closed or read-only.
    """
    self._RaiseIfNotWritable()

    self._WriteAttributeContainer(task_start)


class GZIPStorageMergeReader(interface.FileStorageMergeReader):
  """Class that implements a gzip-based storage file reader for merging."""

  _DATA_BUFFER_SIZE = 1 * 1024 * 1024
  _MAXIMUM_NUMBER_OF_LOCKED_FILE_ATTEMPTS = 4
  _LOCKED_FILE_SLEEP_TIME = 0.5

  def __init__(self, storage_writer, path):
    """Initializes a storage merge reader.

    Args:
      storage_writer (StorageWriter): storage writer.
      path (str): path to the input file.

    Raises:
      IOError: if the input file cannot be opened.
    """
    # On Windows the file can sometimes be in use and we have to wait.
    gzip_file = None
    for attempt in range(1, self._MAXIMUM_NUMBER_OF_LOCKED_FILE_ATTEMPTS):
      try:
        gzip_file = gzip.open(path, 'rb')
        break
      except IOError:
        if attempt == (self._MAXIMUM_NUMBER_OF_LOCKED_FILE_ATTEMPTS - 1):
          raise
        time.sleep(self._LOCKED_FILE_SLEEP_TIME)

    if platform_specific.PlatformIsWindows():
      file_handle = gzip_file.fileno()
      platform_specific.DisableWindowsFileHandleInheritance(file_handle)

    super(GZIPStorageMergeReader, self).__init__(storage_writer)
    self._data_buffer = None
    self._event_data_identifier_mappings = {}
    self._gzip_file = gzip_file
    self._number_of_containers = 0
    self._path = path

  def _AddAttributeContainer(self, attribute_container):
    """Adds a single attribute container to the storage writer.

    Args:
      attribute_container (AttributeContainer): container

    Raises:
      RuntimeError: if the attribute container type is not supported.
    """
    container_type = attribute_container.CONTAINER_TYPE
    if container_type == u'event_source':
      self._storage_writer.AddEventSource(attribute_container)

    elif container_type == u'event_data':
      identifier = attribute_container.GetIdentifier()
      lookup_key = identifier.CopyToString()

      self._storage_writer.AddEventData(attribute_container)

      identifier = attribute_container.GetIdentifier()
      self._event_data_identifier_mappings[lookup_key] = identifier

    elif container_type == u'event':
      if (hasattr(attribute_container, u'event_data_stream_number') and
          hasattr(attribute_container, u'event_data_entry_index')):
        event_data_identifier = identifiers.SerializedStreamIdentifier(
            attribute_container.event_data_stream_number,
            attribute_container.event_data_entry_index)
        lookup_key = event_data_identifier.CopyToString()

        event_data_identifier = self._event_data_identifier_mappings[lookup_key]
        attribute_container.SetEventDataIdentifier(event_data_identifier)

        del attribute_container.event_data_stream_number
        del attribute_container.event_data_entry_index

      # TODO: add event identifier mappings for event tags.

      self._storage_writer.AddEvent(attribute_container)

    elif container_type == u'event_tag':
      event_identifier = identifiers.SerializedStreamIdentifier(
          attribute_container.event_stream_number,
          attribute_container.event_entry_index)
      attribute_container.SetEventIdentifier(event_identifier)

      self._storage_writer.AddEventTag(attribute_container)

    elif container_type == u'extraction_error':
      self._storage_writer.AddError(attribute_container)

    elif container_type == u'analysis_report':
      self._storage_writer.AddAnalysisReport(attribute_container)

    elif container_type not in (u'task_completion', u'task_start'):
      raise RuntimeError(u'Unsupported container type: {0:s}'.format(
          container_type))

  def MergeAttributeContainers(self, maximum_number_of_containers=0):
    """Reads attribute containers from a task storage file into the writer.

    Args:
      maximum_number_of_containers (Optional[int]): maximum number of
          containers to merge, where 0 represent no limit.

    Returns:
      bool: True if the entire task storage file has been merged.

    Raises:
      OSError: if the task storage file cannot be deleted.
    """
    if not self._data_buffer:
      # Do not use gzip.readlines() here since it can consume a large amount
      # of memory.
      self._data_buffer = self._gzip_file.read(self._DATA_BUFFER_SIZE)
      self._number_of_containers = 0

    number_of_containers = 0
    while self._data_buffer:
      lines = self._data_buffer.splitlines(True)
      self._data_buffer = b''
      for index, line in enumerate(lines):
        if not line.endswith(b'\n'):
          self._data_buffer = b''.join(lines[index:])
          continue

        identifier = identifiers.SerializedStreamIdentifier(
            1, self._number_of_containers)

        attribute_container = self._DeserializeAttributeContainer(
            u'attribute_container', line)
        attribute_container.SetIdentifier(identifier)

        self._AddAttributeContainer(attribute_container)

        self._number_of_containers += 1
        number_of_containers += 1

        if (maximum_number_of_containers > 0 and
            number_of_containers >= maximum_number_of_containers):
          self._data_buffer = b''.join(lines[index+1:])
          return False

      additional_data_buffer = self._gzip_file.read(self._DATA_BUFFER_SIZE)
      self._data_buffer = b''.join([self._data_buffer, additional_data_buffer])

    self._gzip_file.close()
    self._gzip_file = None

    # On Windows the file can sometimes be in use and we have to wait.
    for attempt in range(1, self._MAXIMUM_NUMBER_OF_LOCKED_FILE_ATTEMPTS):
      try:
        os.remove(self._path)
        break
      except OSError:
        if attempt == (self._MAXIMUM_NUMBER_OF_LOCKED_FILE_ATTEMPTS - 1):
          raise
        time.sleep(self._LOCKED_FILE_SLEEP_TIME)

    return True


class GZIPStorageFileReader(interface.FileStorageReader):
  """Class that implements a gzip-based storage file reader."""

  def __init__(self, path):
    """Initializes a storage reader.

    Args:
      path (str): path to the input file.
    """
    super(GZIPStorageFileReader, self).__init__(path)
    self._storage_file = GZIPStorageFile()
    self._storage_file.Open(path=path)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the MRUListEx Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import mrulistex

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class TestMRUListExStringPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the string MRUListEx plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        u'MRUlist', key_path=key_path,
        last_written_time=filetime.timestamp, offset=1456)

    # The order is: 201
    value_data = b'\x02\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'MRUListEx', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY, offset=123)
    registry_key.AddValue(registry_value)

    value_data = u'Some random text here'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'0', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1892)
    registry_key.AddValue(registry_value)

    value_data = u'c:\\evil.exe'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'1', data=value_data, data_type=dfwinreg_definitions.REG_BINARY,
        offset=612)
    registry_key.AddValue(registry_value)

    value_data = u'C:\\looks_legit.exe'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'2', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1001)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function."""
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Some Windows\\'
        u'InterestingApp\\MRUlist')
    time_string = u'2012-08-28 09:23:49.002031'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = mrulistex.MRUListExStringPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    self.assertEqual(storage_writer.number_of_events, 1)

    events = list(storage_writer.GetEvents())

    # A MRUListEx event.
    event = events[0]

    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'Index: 1 [MRU Value 2]: C:\\looks_legit.exe '
        u'Index: 2 [MRU Value 0]: Some random text here '
        u'Index: 3 [MRU Value 1]: c:\\evil.exe').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


class TestMRUListExShellItemListPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the shell item list MRUListEx plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\ComDlg32\\OpenSavePidlMRU')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = mrulistex.MRUListExShellItemListPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 65)

    events = list(storage_writer.GetEvents())

    # A MRUListEx event.
    event = events[40]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2011-08-28 22:48:28.159308')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}\\exe] '
        u'Index: 1 [MRU Value 1]: Shell item path: <My Computer> '
        u'P:\\Application Tools\\Firefox 6.0\\Firefox Setup 6.0.exe '
        u'Index: 2 [MRU Value 0]: Shell item path: <Computers and Devices> '
        u'<UNKNOWN: 0x00>\\\\controller\\WebDavShare\\Firefox Setup 3.6.12.exe'
        u'').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # A shell item event.
    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-08 22:16:02')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'Name: ALLOYR~1 '
        u'Long name: Alloy Research '
        u'NTFS file reference: 44518-33 '
        u'Shell item path: <Shared Documents Folder (Users Files)> '
        u'<UNKNOWN: 0x00>\\Alloy Research '
        u'Origin: {0:s}\\*').format(key_path)
    expected_short_message = (
        u'Name: Alloy Research '
        u'NTFS file reference: 44518-33 '
        u'Origin: HKEY_CURRENT_USER\\...')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


class TestMRUListExStringAndShellItemPlugin(test_lib.RegistryPluginTestCase):
  """Tests for the string and shell item MRUListEx plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\RecentDocs')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = mrulistex.MRUListExStringAndShellItemPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 6)

    events = list(storage_writer.GetEvents())

    # A MRUListEx event.
    event = events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-01 13:52:39.113741')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'Index: 1 [MRU Value 17]: Path: The SHIELD, '
        u'Shell item: [The SHIELD.lnk] '
        u'Index: 10 [MRU Value 11]: Path: 5031RR_BalancedLeadership.pdf, '
        u'Shell item: [5031RR_BalancedLeadership.lnk] '
        u'Index: 11 [MRU Value 10]: '
        u'Path: SA-23E Mitchell-Hyundyne Starfury.docx, '
        u'Shell item: [SA-23E Mitchell-Hyundyne Starfury.lnk] '
        u'Index: 12 [MRU Value 9]: Path: StarFury.docx, '
        u'Shell item: [StarFury (3).lnk] '
        u'Index: 13 [MRU Value 6]: Path: StarFury.zip, '
        u'Shell item: [StarFury.lnk] '
        u'Index: 14 [MRU Value 4]: Path: VIBRANIUM.docx, '
        u'Shell item: [VIBRANIUM.lnk] '
        u'Index: 15 [MRU Value 5]: Path: ADAMANTIUM-Background.docx, '
        u'Shell item: [ADAMANTIUM-Background.lnk] '
        u'Index: 16 [MRU Value 3]: Path: Pictures, '
        u'Shell item: [Pictures.lnk] '
        u'Index: 17 [MRU Value 2]: Path: nick_fury_77831.jpg, '
        u'Shell item: [nick_fury_77831.lnk] '
        u'Index: 18 [MRU Value 1]: Path: Downloads, '
        u'Shell item: [Downloads.lnk] '
        u'Index: 19 [MRU Value 0]: Path: wallpaper_medium.jpg, '
        u'Shell item: [wallpaper_medium.lnk] '
        u'Index: 2 [MRU Value 18]: '
        u'Path: captain_america_shield_by_almogrem-d48x9x8.jpg, '
        u'Shell item: [captain_america_shield_by_almogrem-d48x9x8.lnk] '
        u'Index: 3 [MRU Value 16]: Path: captain-america-shield-front.jpg, '
        u'Shell item: [captain-america-shield-front.lnk] '
        u'Index: 4 [MRU Value 12]: Path: Leadership, '
        u'Shell item: [Leadership.lnk] '
        u'Index: 5 [MRU Value 15]: Path: followership.pdf, '
        u'Shell item: [followership.lnk] '
        u'Index: 6 [MRU Value 14]: Path: leaderqualities.pdf, '
        u'Shell item: [leaderqualities.lnk] '
        u'Index: 7 [MRU Value 13]: Path: htlhtl.pdf, '
        u'Shell item: [htlhtl.lnk] '
        u'Index: 8 [MRU Value 8]: Path: StarFury, '
        u'Shell item: [StarFury (2).lnk] '
        u'Index: 9 [MRU Value 7]: Path: Earth_SA-26_Thunderbolt.jpg, '
        u'Shell item: [Earth_SA-26_Thunderbolt.lnk]').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


class TestMRUListExStringAndShellItemListPlugin(
    test_lib.RegistryPluginTestCase):
  """Tests for the string and shell item list MRUListEx plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\ComDlg32\\LastVisitedPidlMRU')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = mrulistex.MRUListExStringAndShellItemListPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 31)

    events = list(storage_writer.GetEvents())

    # A MRUListEx event.
    event = events[30]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-01 13:52:38.966290')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'Index: 1 [MRU Value 1]: Path: chrome.exe, '
        u'Shell item path: <Users Libraries> <UNKNOWN: 0x00> <UNKNOWN: 0x00> '
        u'<UNKNOWN: 0x00> '
        u'Index: 2 [MRU Value 7]: '
        u'Path: {{48E1ED6B-CF49-4609-B1C1-C082BFC3D0B4}}, '
        u'Shell item path: <Shared Documents Folder (Users Files)> '
        u'<UNKNOWN: 0x00>\\Alloy Research '
        u'Index: 3 [MRU Value 6]: '
        u'Path: {{427865A0-03AF-4F25-82EE-10B6CB1DED3E}}, '
        u'Shell item path: <Users Libraries> <UNKNOWN: 0x00> <UNKNOWN: 0x00> '
        u'Index: 4 [MRU Value 5]: '
        u'Path: {{24B5C9BB-48B5-47FF-8343-40481DBA1E2B}}, '
        u'Shell item path: <My Computer> C:\\Users\\nfury\\Documents '
        u'Index: 5 [MRU Value 4]: '
        u'Path: {{0B8CFE96-DB69-4D33-8E3C-36EAB4F709E0}}, '
        u'Shell item path: <My Computer> C:\\Users\\nfury\\Documents\\'
        u'Alloy Research '
        u'Index: 6 [MRU Value 3]: '
        u'Path: {{D4F85F66-003D-4127-BCE9-CAD7A57B2857}}, '
        u'Shell item path: <Users Libraries> <UNKNOWN: 0x00> <UNKNOWN: 0x00> '
        u'Index: 7 [MRU Value 0]: Path: iexplore.exe, '
        u'Shell item path: <My Computer> P:\\Application Tools\\Firefox 6.0 '
        u'Index: 8 [MRU Value 2]: Path: Skype.exe, '
        u'Shell item path: <Users Libraries> <UNKNOWN: 0x00>').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The date filters CLI arguments helper."""

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.filters import file_entry as file_entry_filters
from plaso.lib import errors


class DateFiltersArgumentsHelper(interface.ArgumentsHelper):
  """Date filters CLI arguments helper."""

  NAME = u'date_filters'
  DESCRIPTION = u'Date filters command line arguments.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--date-filter', u'--date_filter', action=u'append', type=str,
        dest=u'date_filters', metavar=u'TYPE_START_END', default=[], help=(
            u'Filter based on file entry date and time ranges. This parameter '
            u'is formatted as "TIME_VALUE,START_DATE_TIME,END_DATE_TIME" where '
            u'TIME_VALUE defines which file entry timestamp the filter applies '
            u'to e.g. atime, ctime, crtime, bkup, etc. START_DATE_TIME and '
            u'END_DATE_TIME define respectively the start and end of the date '
            u'time range. A date time range requires at minimum start or end '
            u'to time of the boundary and END defines the end time. Both '
            u'timestamps be set. The date time values are formatted as: '
            u'YYYY-MM-DD hh:mm:ss.######[+-]##:## Where # are numeric digits '
            u'ranging from 0 to 9 and the seconds fraction can be either 3 '
            u'or 6 digits. The time of day, seconds fraction and timezone '
            u'offset are optional. The default timezone is UTC. E.g. "atime, '
            u'2013-01-01 23:12:14, 2013-02-23". This parameter can be repeated '
            u'as needed to add additional date boundaries, e.g. once for '
            u'atime, once for crtime, etc.'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          u'Configuration object is not an instance of CLITool')

    filter_collection = getattr(
        configuration_object, u'_filter_collection', None)
    if not filter_collection:
      raise errors.BadConfigObject(
          u'Filter collection missing from configuration object')

    date_filters = getattr(options, u'date_filters', None)
    if not date_filters:
      return

    file_entry_filter = file_entry_filters.DateTimeFileEntryFilter()

    for date_filter in date_filters:
      date_filter_pieces = date_filter.split(u',')
      if len(date_filter_pieces) != 3:
        raise errors.BadConfigOption(
            u'Badly formed date filter: {0:s}'.format(date_filter))

      time_value, start_time_string, end_time_string = date_filter_pieces
      time_value = time_value.strip()
      start_time_string = start_time_string.strip()
      end_time_string = end_time_string.strip()

      try:
        file_entry_filter.AddDateTimeRange(
            time_value, start_time_string=start_time_string,
            end_time_string=end_time_string)
      except ValueError:
        raise errors.BadConfigOption(
            u'Badly formed date filter: {0:s}'.format(date_filter))

    filter_collection.AddFilter(file_entry_filter)


manager.ArgumentHelperManager.RegisterHelper(DateFiltersArgumentsHelper)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the output mediator object."""

import unittest

from plaso.containers import events
from plaso.engine import knowledge_base
from plaso.formatters import interface as formatters_interface
from plaso.formatters import manager as formatters_manager
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import mediator


class TestEvent(events.EventObject):
  """Test event object."""
  DATA_TYPE = u'test:mediator'

  def __init__(self):
    """Initializes an event object."""
    super(TestEvent, self).__init__()
    self.timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01')
    self.timestamp_desc = definitions.TIME_DESCRIPTION_CHANGE
    self.hostname = u'ubuntu'
    self.filename = u'log/syslog.1'
    self.text = (
        u'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
        u'closed for user root)')
    self.username = u'root'


class TestEventFormatter(formatters_interface.EventFormatter):
  """Test event formatter."""
  DATA_TYPE = u'test:mediator'
  FORMAT_STRING = u'{text}'

  SOURCE_SHORT = 'LOG'
  SOURCE_LONG = 'Syslog'


class OutputMediatorTest(unittest.TestCase):
  """Tests for the output mediator object."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    knowledge_base_object = knowledge_base.KnowledgeBase()
    self._output_mediator = mediator.OutputMediator(
        knowledge_base_object, None)

  def testGetEventFormatter(self):
    """Tests the GetEventFormatter function."""
    event_object = TestEvent()

    formatters_manager.FormattersManager.RegisterFormatter(
        TestEventFormatter)

    event_formatter = self._output_mediator.GetEventFormatter(event_object)
    self.assertIsInstance(event_formatter, TestEventFormatter)

    formatters_manager.FormattersManager.DeregisterFormatter(
        TestEventFormatter)

  def testGetFormattedMessages(self):
    """Tests the GetFormattedMessages function."""
    event_object = TestEvent()

    formatters_manager.FormattersManager.RegisterFormatter(
        TestEventFormatter)

    expected_message = (
        u'Reporter <CRON> PID: 8442'
        u' (pam_unix(cron:session): session closed for user root)')

    message, message_short = self._output_mediator.GetFormattedMessages(
        event_object)
    self.assertEqual(message, expected_message)
    self.assertEqual(message_short, expected_message)

    formatters_manager.FormattersManager.DeregisterFormatter(
        TestEventFormatter)

  def testGetFormattedSources(self):
    """Tests the GetFormattedSources function."""
    event_object = TestEvent()

    formatters_manager.FormattersManager.RegisterFormatter(
        TestEventFormatter)

    source_short, source = self._output_mediator.GetFormattedSources(
        event_object)
    self.assertEqual(source, u'Syslog')
    self.assertEqual(source_short, u'LOG')

    formatters_manager.FormattersManager.DeregisterFormatter(
        TestEventFormatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_object = TestEvent()

    formatters_manager.FormattersManager.RegisterFormatter(
        TestEventFormatter)

    expected_attribute_names = set([u'text'])

    attribute_names = self._output_mediator.GetFormatStringAttributeNames(
        event_object)
    self.assertEqual(attribute_names, expected_attribute_names)

    formatters_manager.FormattersManager.DeregisterFormatter(
        TestEventFormatter)

  def testGetHostname(self):
    """Tests the GetHostname function."""
    event_object = TestEvent()

    hostname = self._output_mediator.GetHostname(event_object)
    self.assertEqual(hostname, u'ubuntu')

  def testGetMACBRepresentation(self):
    """Tests the GetMACBRepresentation function."""
    event_object = TestEvent()

    macb_representation = self._output_mediator.GetMACBRepresentation(
        event_object)
    self.assertEqual(macb_representation, u'..C.')

  def testGetStoredHostname(self):
    """Tests the GetStoredHostname function."""
    stored_hostname = self._output_mediator.GetStoredHostname()
    self.assertIsNone(stored_hostname)

  def testGetUsername(self):
    """Tests the GetUsername function."""
    event_object = TestEvent()

    username = self._output_mediator.GetUsername(event_object)
    self.assertEqual(username, u'root')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the windows services analysis plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfvfs.path import fake_path_spec

from plaso.analysis import windows_services
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import winreg

from tests import test_lib as shared_test_lib
from tests.analysis import test_lib


class WindowsServicesTest(test_lib.AnalysisPluginTestCase):
  """Tests for the Windows Services analysis plugin."""

  _TEST_EVENTS = [
      {u'key_path': u'\\ControlSet001\\services\\TestbDriver',
       u'regvalue': {u'ImagePath': u'C:\\Dell\\testdriver.sys', u'Type': 2,
                     u'Start': 2, u'ObjectName': u''},
       u'timestamp': 1346145829002031},
      # This is almost the same, but different timestamp and source, so that
      # we can test the service de-duplication.
      {u'key_path': u'\\ControlSet003\\services\\TestbDriver',
       u'regvalue': {u'ImagePath': u'C:\\Dell\\testdriver.sys', u'Type': 2,
                     u'Start': 2, u'ObjectName': u''},
       u'timestamp': 1346145839002031},
  ]

  def _CreateTestEventObject(self, event_dictionary):
    """Create a test event with a set of attributes.

    Args:
      event_dictionary (dict[str, str]): contains attributes of an event to add
          to the queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    date_time = dfdatetime_filetime.Filetime(
        timestamp=event_dictionary[u'timestamp'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_WRITTEN)
    event.data_type = u'windows:registry:service'

    for attribute_name, attribute_value in event_dictionary.items():
      setattr(event, attribute_name, attribute_value)

    return event

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    events = []
    for event_dictionary in self._TEST_EVENTS:
      event_dictionary[u'pathspec'] = fake_path_spec.FakePathSpec(
          location=u'C:\\WINDOWS\\system32\\SYSTEM')

      event = self._CreateTestEventObject(event_dictionary)
      events.append(event)

    plugin = windows_services.WindowsServicesAnalysisPlugin()
    storage_writer = self._AnalyzeEvents(events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)

    analysis_report = storage_writer.analysis_reports[0]

    expected_text = (
        u'Listing Windows Services\n'
        u'TestbDriver\n'
        u'\tImage Path    = C:\\Dell\\testdriver.sys\n'
        u'\tService Type  = File System Driver (0x2)\n'
        u'\tStart Type    = Auto Start (2)\n'
        u'\tService Dll   = \n'
        u'\tObject Name   = \n'
        u'\tSources:\n'
        u'\t\tC:\\WINDOWS\\system32\\SYSTEM:'
        u'\\ControlSet001\\services\\TestbDriver\n'
        u'\t\tC:\\WINDOWS\\system32\\SYSTEM:'
        u'\\ControlSet003\\services\\TestbDriver\n\n')

    self.assertEqual(expected_text, analysis_report.text)
    self.assertEqual(analysis_report.plugin_name, 'windows_services')

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testExamineEventAndCompileReportOnSystemFile(self):
    """Tests the ExamineEvent and CompileReport functions on a SYSTEM file."""
    # We could remove the non-Services plugins, but testing shows that the
    # performance gain is negligible.

    parser = winreg.WinRegistryParser()
    plugin = windows_services.WindowsServicesAnalysisPlugin()

    storage_writer = self._ParseAndAnalyzeFile([u'SYSTEM'], parser, plugin)

    self.assertEqual(storage_writer.number_of_events, 31436)

    self.assertEqual(len(storage_writer.analysis_reports), 1)

    analysis_report = storage_writer.analysis_reports[0]

    # We'll check that a few strings are in the report, like they're supposed
    # to be, rather than checking for the exact content of the string,
    # as that's dependent on the full path to the test files.
    test_strings = [
        u'1394ohci',
        u'WwanSvc',
        u'Sources:',
        u'ControlSet001',
        u'ControlSet002']

    for string in test_strings:
      self.assertIn(string, analysis_report.text)

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testExamineEventAndCompileReportOnSystemFileWithYAML(self):
    """Tests the ExamineEvent and CompileReport with YAML."""
    # We could remove the non-Services plugins, but testing shows that the
    # performance gain is negligible.

    parser = winreg.WinRegistryParser()
    plugin = windows_services.WindowsServicesAnalysisPlugin()
    plugin.SetOutputFormat(u'yaml')

    storage_writer = self._ParseAndAnalyzeFile([u'SYSTEM'], parser, plugin)

    self.assertEqual(storage_writer.number_of_events, 31436)

    self.assertEqual(len(storage_writer.analysis_reports), 1)

    analysis_report = storage_writer.analysis_reports[0]

    # We'll check that a few strings are in the report, like they're supposed
    # to be, rather than checking for the exact content of the string,
    # as that's dependent on the full path to the test files.
    test_strings = [
        windows_services.WindowsService.yaml_tag,
        u'1394ohci',
        u'WwanSvc',
        u'ControlSet001',
        u'ControlSet002']

    for string in test_strings:
      self.assertIn(string, analysis_report.text)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The 4n6time MySQL database output module CLI arguments helper."""

from plaso.lib import errors
from plaso.cli.helpers import interface
from plaso.cli.helpers import database_config
from plaso.cli.helpers import shared_4n6time_output
from plaso.cli.helpers import manager
from plaso.output import mysql_4n6time


class MySQL4n6TimeDatabaseArgumentsHelper(
    database_config.DatabaseArgumentsHelper):
  """4n6time MySQL database server CLI arguments helper."""

  _DEFAULT_USERNAME = u'root'
  _DEFAULT_PASSWORD = u'forensic'


class MySQL4n6TimeOutputArgumentsHelper(interface.ArgumentsHelper):
  """4n6time MySQL database output module CLI arguments helper."""

  NAME = u'4n6time_mysql'
  CATEGORY = u'output'
  DESCRIPTION = u'Argument helper for the 4n6Time MySQL output module.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    shared_4n6time_output.Shared4n6TimeOutputArgumentsHelper.AddArguments(
        argument_group)
    MySQL4n6TimeDatabaseArgumentsHelper.AddArguments(argument_group)

  @classmethod
  def ParseOptions(cls, options, output_module):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      output_module (OutputModule): output module to configure.

    Raises:
      BadConfigObject: when the output module object is of the wrong type.
    """
    if not isinstance(output_module, mysql_4n6time.MySQL4n6TimeOutputModule):
      raise errors.BadConfigObject(
          u'Output module is not an instance of MySQL4n6TimeOutputModule')

    MySQL4n6TimeDatabaseArgumentsHelper.ParseOptions(options, output_module)
    shared_4n6time_output.Shared4n6TimeOutputArgumentsHelper.ParseOptions(
        options, output_module)


manager.ArgumentHelperManager.RegisterHelper(MySQL4n6TimeOutputArgumentsHelper)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Zsh extended_history formatter."""

import unittest

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import time_events
from plaso.formatters import zsh_extended_history
from plaso.lib import definitions

from tests.formatters import test_lib


class ZshExtendedHistoryFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Zsh extended_history event formatter."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    self._formatter = zsh_extended_history.ZshExtendedHistoryEventFormatter()

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    expected_attribute_names = [
        u'command',
        u'elapsed_seconds']

    self._TestGetFormatStringAttributeNames(
        self._formatter, expected_attribute_names)

  def testGetMessages(self):
    """Tests the GetMessages method."""
    date_time = dfdatetime_posix_time.PosixTime(timestamp=1457771210)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
    event.command = u'cd plaso'
    event.data_type = u'shell:zsh:history'
    event.elapsed_seconds = 0

    expected_messages = (u'cd plaso Time elapsed: 0 seconds', u'cd plaso')
    messages = self._formatter.GetMessages(None, event)
    self.assertEqual(messages, expected_messages)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Windows Firewall Log file."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import manager
from plaso.parsers import text_parser

import pytz  # pylint: disable=wrong-import-order


class WinFirewallEventData(events.EventData):
  """Windows Firewall event data.

  Attributes:
    action (str): action taken.
    protocol (str): IP protocol.
    source_ip (str): source IP address.
    dest_ip (str): destination IP address.
    source_port (str): TCP or UDP source port.
    dest_port (str): TCP or UDP destination port.
    size (str): size of ???
    flags (str): TCP flags.
    tcp_seq (str): TCP sequence number.
    tcp_ack (str): TCP ACK ???
    tcp_win (str): TCP window size ???
    icmp_type (str): ICMP type.
    icmp_code (str): ICMP code.
    info (str): ???
    path (str): ???
  """

  DATA_TYPE = u'windows:firewall:log_entry'

  def __init__(self):
    """Initializes event data."""
    super(WinFirewallEventData, self).__init__(data_type=self.DATA_TYPE)
    self.action = None
    self.dest_ip = None
    self.dest_port = None
    self.flags = None
    self.icmp_code = None
    self.icmp_type = None
    self.info = None
    self.path = None
    self.protocol = None
    self.size = None
    self.source_ip = None
    self.source_port = None
    self.tcp_ack = None
    self.tcp_seq = None
    self.tcp_win = None


class WinFirewallParser(text_parser.PyparsingSingleLineTextParser):
  """Parses the Windows Firewall Log file."""

  NAME = u'winfirewall'
  DESCRIPTION = u'Parser for Windows Firewall Log files.'

  # TODO: Add support for custom field names. Currently this parser only
  # supports the default fields, which are:
  #   date time action protocol src-ip dst-ip src-port dst-port size
  #   tcpflags tcpsyn tcpack tcpwin icmptype icmpcode info path

  _BLANK = pyparsing.Suppress(pyparsing.Literal(u'-'))

  _WORD = (
      pyparsing.Word(pyparsing.alphanums, min=1) |
      pyparsing.Word(pyparsing.alphanums + u'-', min=2) |
      _BLANK)

  _INTEGER = (
      pyparsing.Word(pyparsing.nums, min=1).setParseAction(
          text_parser.ConvertTokenToInteger) |
      _BLANK)

  _IP_ADDRESS = (
      text_parser.PyparsingConstants.IPV4_ADDRESS |
      text_parser.PyparsingConstants.IPV6_ADDRESS |
      _BLANK)

  _PORT_NUMBER = (
      pyparsing.Word(pyparsing.nums, min=1, max=6).setParseAction(
          text_parser.ConvertTokenToInteger) |
      _BLANK)

  _LOG_LINE = (
      text_parser.PyparsingConstants.DATE_TIME.setResultsName(u'date_time') +
      _WORD.setResultsName(u'action') +
      _WORD.setResultsName(u'protocol') +
      _IP_ADDRESS.setResultsName(u'source_ip') +
      _IP_ADDRESS.setResultsName(u'dest_ip') +
      _PORT_NUMBER.setResultsName(u'source_port') +
      _INTEGER.setResultsName(u'dest_port') +
      _INTEGER.setResultsName(u'size') +
      _WORD.setResultsName(u'flags') +
      _INTEGER.setResultsName(u'tcp_seq') +
      _INTEGER.setResultsName(u'tcp_ack') +
      _INTEGER.setResultsName(u'tcp_win') +
      _INTEGER.setResultsName(u'icmp_type') +
      _INTEGER.setResultsName(u'icmp_code') +
      _WORD.setResultsName(u'info') +
      _WORD.setResultsName(u'path'))

  LINE_STRUCTURES = [
      (u'comment', text_parser.PyparsingConstants.COMMENT_LINE_HASH),
      (u'logline', _LOG_LINE),
  ]

  def __init__(self):
    """Initializes a parser object."""
    super(WinFirewallParser, self).__init__()
    self._software = None
    self._use_local_timezone = False
    self._version = None

  def _ParseCommentRecord(self, structure):
    """Parse a comment and store appropriate attributes.

    Args:
      structure: A pyparsing.ParseResults object from a line in the
                 log file.
    """
    comment = structure[1]
    if comment.startswith(u'Version'):
      _, _, self._version = comment.partition(u':')
    elif comment.startswith(u'Software'):
      _, _, self._software = comment.partition(u':')
    elif comment.startswith(u'Time'):
      _, _, time_format = comment.partition(u':')
      if u'local' in time_format.lower():
        self._use_local_timezone = True

  def _ParseLogLine(self, parser_mediator, structure):
    """Parse a single log line and and produce an event object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.
    """
    try:
      date_time = dfdatetime_time_elements.TimeElements(
          time_elements_tuple=structure.date_time)
      date_time.is_local_time = True
    except ValueError:
      parser_mediator.ProduceExtractionError(
          u'invalid date time value: {0!s}'.format(structure.date_time))
      return

    event_data = WinFirewallEventData()
    event_data.action = structure.action
    event_data.dest_ip = structure.dest_ip
    event_data.dest_port = structure.dest_port
    event_data.flags = structure.flags
    event_data.icmp_code = structure.icmp_code
    event_data.icmp_type = structure.icmp_type
    event_data.info = structure.info
    event_data.path = structure.path
    event_data.protocol = structure.protocol
    event_data.size = structure.size
    event_data.source_ip = structure.source_ip
    event_data.source_port = structure.source_port
    event_data.tcp_ack = structure.tcp_ack
    event_data.tcp_seq = structure.tcp_seq
    event_data.tcp_win = structure.tcp_win

    if self._use_local_timezone:
      time_zone = parser_mediator.timezone
    else:
      time_zone = pytz.UTC

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_WRITTEN, time_zone=time_zone)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses a log record structure and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      key (str): identifier of the structure of tokens.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.

    Raises:
      ParseError: when the structure type is unknown.
    """
    if key not in (u'comment', u'logline'):
      raise errors.ParseError(
          u'Unable to parse record, unknown structure: {0:s}'.format(key))

    if key == u'comment':
      self._ParseCommentRecord(structure)

    elif key == u'logline':
      self._ParseLogLine(parser_mediator, structure)

  def VerifyStructure(self, unused_parser_mediator, line):
    """Verify that this file is a firewall log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      line (bytes): line from a text file.

    Returns:
      bool: True if the line is in the expected format, False if not.
    """
    # TODO: Examine other versions of the file format and if this parser should
    # support them.
    return line == b'#Version: 1.5'


manager.ParsersManager.RegisterParser(WinFirewallParser)



# -*- coding: utf-8 -*-
"""This file contains the Bencode Parser.

Plaso's engine calls BencodeParser when it encounters bencoded files to be
processed, typically seen for BitTorrent data.
"""

import logging
import re
import os

import bencode

from plaso.lib import errors
from plaso.parsers import interface
from plaso.parsers import manager


class BencodeParser(interface.FileObjectParser):
  """Deserializes bencoded file; produces a dictionary containing bencoded data.

  The Plaso engine calls parsers by their Parse() method. The Parse() function
  deserializes bencoded files using the BitTorrent-bencode library and
  calls plugins (BencodePlugin) registered through the interface by their
  Process() to produce event objects.

  Plugins are how this parser understands the content inside a bencoded file,
  each plugin holds logic specific to a particular bencoded file. See the
  bencode_plugins / directory for examples of how bencode plugins are
  implemented.
  """

  _INITIAL_FILE_OFFSET = None

  # Regex match for a bencode dictionary followed by a field size.
  BENCODE_RE = re.compile(r'd[0-9]')

  NAME = u'bencode'
  DESCRIPTION = u'Parser for bencoded files.'

  _plugin_classes = {}

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a bencoded file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    file_object.seek(0, os.SEEK_SET)
    header = file_object.read(2)
    if not self.BENCODE_RE.match(header):
      raise errors.UnableToParseFile(u'Not a valid Bencoded file.')

    file_object.seek(0, os.SEEK_SET)
    try:
      data_object = bencode.bdecode(file_object.read())

    except (IOError, bencode.BTFailure) as exception:
      raise errors.UnableToParseFile(
          u'[{0:s}] unable to parse file: {1:s} with error: {2:s}'.format(
              self.NAME, parser_mediator.GetDisplayName(), exception))

    if not data_object:
      raise errors.UnableToParseFile(
          u'[{0:s}] missing decoded data for file: {1:s}'.format(
              self.NAME, parser_mediator.GetDisplayName()))

    for plugin in self._plugins:
      try:
        plugin.UpdateChainAndProcess(parser_mediator, data=data_object)
      except errors.WrongBencodePlugin as exception:
        logging.debug(u'[{0:s}] wrong plugin: {1:s}'.format(
            self.NAME, exception))


manager.ParsersManager.RegisterParser(BencodeParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome extension activity database plugin."""

import unittest

from plaso.formatters import chrome_extension_activity  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import chrome_extension_activity

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class ChromeExtensionActivityPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome extension activity database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'Extension Activity'])
  def testProcess(self):
    """Tests the Process function on a Chrome extension activity database."""
    plugin = chrome_extension_activity.ChromeExtensionActivityPlugin()
    cache = sqlite.SQLiteCache()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'Extension Activity'], plugin, cache=cache)

    self.assertEqual(storage_writer.number_of_events, 56)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_UNKNOWN)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-25 21:08:23.698737')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_extension_id = u'ognampngfcbddbfemdapefohjiobgbdl'
    self.assertEqual(event.extension_id, expected_extension_id)

    self.assertEqual(event.action_type, 1)
    self.assertEqual(event.activity_id, 48)
    self.assertEqual(event.api_name, u'browserAction.onClicked')

    expected_message = (
        u'Chrome extension: ognampngfcbddbfemdapefohjiobgbdl '
        u'Action type: API event callback (type 1) '
        u'Activity identifier: 48 '
        u'API name: browserAction.onClicked')
    expected_short_message = (
        u'ognampngfcbddbfemdapefohjiobgbdl browserAction.onClicked')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests the multi-process processing engine."""

import os
import unittest

from artifacts import reader as artifacts_reader
from artifacts import registry as artifacts_registry
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.containers import sessions
from plaso.engine import configurations
from plaso.multi_processing import task_engine
from plaso.storage import zip_file as storage_zip_file

from tests import test_lib as shared_test_lib


class TaskMultiProcessEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the task multi-process engine."""

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testProcessSources(self):
    """Tests the PreprocessSources and ProcessSources function."""
    registry = artifacts_registry.ArtifactDefinitionsRegistry()
    reader = artifacts_reader.YamlArtifactsReader()
    path = shared_test_lib.GetTestFilePath([u'artifacts'])
    registry.ReadFromDirectory(reader, path)

    test_engine = task_engine.TaskMultiProcessEngine(
        maximum_number_of_tasks=100)

    source_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=source_path)
    source_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=os_path_spec)

    test_engine.PreprocessSources(registry, [source_path_spec])

    session = sessions.Session()

    configuration = configurations.ProcessingConfiguration()
    configuration.parser_filter_expression = u'filestat'

    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file = os.path.join(temp_directory, u'storage.plaso')
      storage_writer = storage_zip_file.ZIPStorageFileWriter(
          session, temp_file)

      test_engine.ProcessSources(
          session.identifier, [source_path_spec], storage_writer, configuration)

    # TODO: implement a way to obtain the results without relying
    # on multi-process primitives e.g. by writing to a file.
    # self.assertEqual(storage_writer.number_of_events, 15)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains functions for certain platform specific operations."""
import sys


# Windows-only imports
try:
  import msvcrt
  import win32api
  import win32con
except ImportError:
  msvcrt = None
  win32api = None
  win32con = None


def DisableWindowsFileHandleInheritance(file_descriptor):
  """Flags a Windows file descriptor so that child processes don't inherit it.

  Args:
    file_descriptor (int): file handle descriptor, as returned by fileno() and
        similar methods.
  """
  if msvcrt and win32api and win32con:
    os_handle = msvcrt.get_osfhandle(file_descriptor)
    win32api.SetHandleInformation(os_handle, win32con.HANDLE_FLAG_INHERIT, 0)
    return
  raise RuntimeError


def PlatformIsDarwin():
  """Checks if the current platform is Windows.

  Returns:
    bool: True if Python is running on Darwin.
  """
  return sys.platform.startswith(u'darwin')


def PlatformIsLinux():
  """Checks if the current platform is Windows.

  Returns:
    bool: True if Python is running on Windows.
  """
  return sys.platform.startswith(u'linux')


def PlatformIsWindows():
  """Checks if the current platform is Windows.

  Returns:
    bool: True if Python is running on Windows.
  """
  return sys.platform.startswith(u'win') or sys.platform.startswith(u'cygwin')



# -*- coding: utf-8 -*-
"""The default plugin for parsing OLE Compound Files (OLECF)."""

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import olecf
from plaso.parsers.olecf_plugins import interface


class OLECFItemEvent(time_events.DateTimeValuesEvent):
  """Convenience class for an OLECF item event."""

  DATA_TYPE = u'olecf:item'

  def __init__(self, date_time, date_time_description, olecf_item):
    """Initializes an event.

    Args:
      date_time (dfdatetime.DateTimeValues): date and time values.
      date_time_description (str): description of the meaning of the date
          and time values.
      olecf_item (pyolecf.item): OLECF item.
    """
    super(OLECFItemEvent, self).__init__(date_time, date_time_description)
    self.name = olecf_item.name

    # TODO: need a better way to express the original location of the
    # original data.
    self.offset = 0

    # TODO: have pyolecf return the item type here.
    # self.type = olecf_item.type
    self.size = olecf_item.size


class DefaultOLECFPlugin(interface.OLECFPlugin):
  """Class to define the default OLECF file plugin."""

  NAME = u'olecf_default'
  DESCRIPTION = u'Parser for a generic OLECF item.'

  def _ParseItem(self, parser_mediator, olecf_item):
    """Parses an OLECF item.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      olecf_item (pyolecf.item): OLECF item.

    Returns:
      bool: True if an event was produced.
    """
    event = None
    result = False

    creation_time, modification_time = self._GetTimestamps(olecf_item)

    if creation_time:
      date_time = dfdatetime_filetime.Filetime(timestamp=creation_time)
      event = OLECFItemEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION, olecf_item)
      parser_mediator.ProduceEvent(event)

    if modification_time:
      date_time = dfdatetime_filetime.Filetime(timestamp=modification_time)
      event = OLECFItemEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION, olecf_item)
      parser_mediator.ProduceEvent(event)

    if event:
      result = True

    for sub_item in olecf_item.sub_items:
      if self._ParseItem(parser_mediator, sub_item):
        result = True

    return result

  def Process(self, parser_mediator, root_item=None, **kwargs):
    """Parses an OLECF file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      root_item (Optional[pyolecf.item]): root item of the OLECF file.

    Raises:
      ValueError: If the root item is not set.
    """
    # This will raise if unhandled keyword arguments are passed.
    super(DefaultOLECFPlugin, self).Process(parser_mediator, **kwargs)

    if not root_item:
      raise ValueError(u'Root item not set.')

    if not self._ParseItem(parser_mediator, root_item):
      # If no event was produced, produce at least one for the root item.
      date_time = dfdatetime_semantic_time.SemanticTime(u'Not set')
      event = OLECFItemEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION, root_item)
      parser_mediator.ProduceEvent(event)


olecf.OLECFParser.RegisterPlugin(DefaultOLECFPlugin)



# -*_ coding: utf-8 -*-
"""Parser for SCCM Logs."""
import re

import pyparsing

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import manager
from plaso.parsers import text_parser


class SCCMLogEventData(events.EventData):
  """SCCM log event data.

  Attributes:
    component (str): component.
    text (str): text.
  """

  DATA_TYPE = u'software_management:sccm:log'

  def __init__(self):
    """Initializes event data."""
    super(SCCMLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.component = None
    self.text = None


class SCCMParser(text_parser.PyparsingMultiLineTextParser):
  """Parser for Windows System Center Configuration Manager (SCCM) logs."""

  NAME = u'sccm'
  DESCRIPTION = u'Parser for SCCM logs files.'

  _ENCODING = u'utf-8-sig'

  # Increasing the buffer size as SCCM messages are commonly well larger
  # than the default value.
  BUFFER_SIZE = 16384

  LINE_STRUCTURES = []

  _MICRO_SECONDS_PER_MINUTE = 60 * 1000000

  _FOUR_DIGITS = text_parser.PyparsingConstants.FOUR_DIGITS
  _ONE_OR_TWO_DIGITS = text_parser.PyparsingConstants.ONE_OR_TWO_DIGITS

  # PyParsing Components used to construct grammars for parsing lines.
  _PARSING_COMPONENTS = {
      u'msg_left_delimiter': pyparsing.Literal(u'<![LOG['),
      u'msg_right_delimiter': pyparsing.Literal(u']LOG]!><time="'),
      u'year': _FOUR_DIGITS.setResultsName(u'year'),
      u'month': _ONE_OR_TWO_DIGITS.setResultsName(u'month'),
      u'day': _ONE_OR_TWO_DIGITS.setResultsName(u'day'),
      u'microsecond': pyparsing.Regex(r'\d{3,7}').
                      setResultsName(u'microsecond'),
      u'utc_offset_minutes': pyparsing.Regex(r'[-+]\d{3}').
                             setResultsName(u'utc_offset_minutes'),
      u'date_prefix': pyparsing.Literal(u'" date="').
                      setResultsName(u'date_prefix'),
      u'component_prefix': pyparsing.Literal(u'" component="').
                           setResultsName(u'component_prefix'),
      u'component': pyparsing.Word(pyparsing.alphanums).
                    setResultsName(u'component'),
      u'text': pyparsing.Regex(r'.*?(?=(]LOG]!><time="))', re.DOTALL).
               setResultsName(u'text'),
      u'line_remainder': pyparsing.Regex(r'.*?(?=(\<!\[LOG\[))', re.DOTALL).
                         setResultsName(u'line_remainder'),
      u'lastline_remainder': pyparsing.restOfLine.
                             setResultsName(u'lastline_remainder'),
      u'hour': _ONE_OR_TWO_DIGITS.setResultsName(u'hour'),
      u'minute': text_parser.PyparsingConstants.TWO_DIGITS.
                 setResultsName(u'minute'),
      u'second': text_parser.PyparsingConstants.TWO_DIGITS.
                 setResultsName(u'second')}

  # Base grammar for individual log event lines.
  LINE_GRAMMAR_BASE = (
      _PARSING_COMPONENTS[u'msg_left_delimiter'] +
      _PARSING_COMPONENTS[u'text'] +
      _PARSING_COMPONENTS[u'msg_right_delimiter'] +
      _PARSING_COMPONENTS[u'hour'] +
      pyparsing.Suppress(u':') + _PARSING_COMPONENTS[u'minute'] +
      pyparsing.Suppress(u':') + _PARSING_COMPONENTS[u'second'] +
      pyparsing.Suppress(u'.') + _PARSING_COMPONENTS[u'microsecond'] +
      _PARSING_COMPONENTS[u'date_prefix'] + _PARSING_COMPONENTS[u'month'] +
      pyparsing.Suppress(u'-') + _PARSING_COMPONENTS[u'day'] +
      pyparsing.Suppress(u'-') + _PARSING_COMPONENTS[u'year'] +
      _PARSING_COMPONENTS[u'component_prefix'] +
      _PARSING_COMPONENTS[u'component'])

  # Grammar for individual log event lines with a minutes offset from UTC.
  LINE_GRAMMAR_OFFSET = (
      _PARSING_COMPONENTS[u'msg_left_delimiter'] +
      _PARSING_COMPONENTS[u'text'] +
      _PARSING_COMPONENTS[u'msg_right_delimiter'] +
      _PARSING_COMPONENTS[u'hour'] +
      pyparsing.Suppress(u':') + _PARSING_COMPONENTS[u'minute'] +
      pyparsing.Suppress(u':') + _PARSING_COMPONENTS[u'second'] +
      pyparsing.Suppress(u'.') + _PARSING_COMPONENTS[u'microsecond'] +
      _PARSING_COMPONENTS[u'utc_offset_minutes'] +
      _PARSING_COMPONENTS[u'date_prefix'] + _PARSING_COMPONENTS[u'month'] +
      pyparsing.Suppress(u'-') + _PARSING_COMPONENTS[u'day'] +
      pyparsing.Suppress(u'-') + _PARSING_COMPONENTS[u'year'] +
      _PARSING_COMPONENTS[u'component_prefix'] +
      _PARSING_COMPONENTS[u'component'])

  LINE_STRUCTURES = [
      (u'log_entry',
       LINE_GRAMMAR_BASE + _PARSING_COMPONENTS[u'line_remainder']),
      (u'log_entry_at_end',
       LINE_GRAMMAR_BASE +_PARSING_COMPONENTS[u'lastline_remainder'] +
       pyparsing.lineEnd),
      (u'log_entry_offset',
       LINE_GRAMMAR_OFFSET + _PARSING_COMPONENTS[u'line_remainder']),
      (u'log_entry_offset_at_end',
       LINE_GRAMMAR_OFFSET + _PARSING_COMPONENTS[u'lastline_remainder'] +
       pyparsing.lineEnd)]

  def ParseRecord(self, parser_mediator, key, structure):
    """Parse the record and return an SCCM log event object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.
      structure (pyparsing.ParseResults): structure of tokens derived from
          a line of a text file.

    Raises:
      ParseError: when the structure type is unknown.
      TimestampError: when a non-int value for microseconds is encountered.
    """
    if key not in (
        u'log_entry', u'log_entry_at_end', u'log_entry_offset',
        u'log_entry_offset_at_end'):
      raise errors.ParseError(
          u'Unable to parse record, unknown structure: {0:s}'.format(key))

    # Sometimes, SCCM logs will exhibit a seven-digit sub-second precision
    # (100 nanosecond intervals). Using six-digit precision because
    # timestamps are in microseconds.
    if len(structure.microsecond) > 6:
      structure.microsecond = structure.microsecond[0:6]

    try:
      microseconds = int(structure.microsecond, 10)
    except ValueError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to determine microseconds with error: {0:s}'.format(
              exception))
      return

    # 3-digit precision is milliseconds,
    # so multiply by 1000 to convert to microseconds
    if len(structure.microsecond) == 3:
      microseconds *= 1000

    try:
      timestamp = timelib.Timestamp.FromTimeParts(
          structure.year, structure.month, structure.day,
          structure.hour, structure.minute, structure.second, microseconds)
    except errors.TimestampError as exception:
      timestamp = timelib.Timestamp.NONE_TIMESTAMP
      parser_mediator.ProduceExtractionError(
          u'unable to determine timestamp with error: {0:s}'.format(
              exception))

    # If an offset is given for the event, apply the offset to convert to UTC.
    if timestamp and u'offset' in key:
      try:
        delta_microseconds = int(structure.utc_offset_minutes[1:], 10)
      except (IndexError, ValueError) as exception:
        raise errors.TimestampError(
            u'Unable to parse minute offset from UTC with error: {0:s}.'.format(
                exception))

      delta_microseconds *= self._MICRO_SECONDS_PER_MINUTE
      if structure.utc_offset_minutes[0] == u'-':
        delta_microseconds = -delta_microseconds
      timestamp += delta_microseconds

    event_data = SCCMLogEventData()
    event_data.component = structure.component
    # TODO: pass line number to offset or remove.
    event_data.offset = 0
    event_data.text = structure.text

    event = time_events.TimestampEvent(
        timestamp, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, parser_mediator, lines):
    """Verifies whether content corresponds to an SCCM log file.

    Args:
      parser_mediator: A parser mediator object (instance of ParserMediator).
      lines: A buffer containing lines from a log file.

    Returns:
      Returns a boolean that indicates the log lines appear to contain
      content from an SCCM log file.
    """
    # Identify the token to which we attempt a match.
    match = self._PARSING_COMPONENTS[u'msg_left_delimiter'].match

    # Because logs files can lead with a partial event,
    # we can't assume that the first character (post-BOM)
    # in the file is the beginning of our match - so we
    # look for match anywhere in lines.
    return match in lines


manager.ParsersManager.RegisterParser(SCCMParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the attribute container manager."""

import unittest

from plaso.containers import manager

from tests import test_lib as shared_test_lib
from tests.containers import test_lib


class AttributeContainersManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the attribute container manager."""

  def testAttributeContainerRegistration(self):
    """Tests the Register and DeregisterAttributeContainer functions."""
    # pylint: disable=protected-access
    number_of_classes = len(
        manager.AttributeContainersManager._attribute_container_classes)

    manager.AttributeContainersManager.RegisterAttributeContainer(
        test_lib.TestAttributeContainer)
    self.assertEqual(
        len(manager.AttributeContainersManager._attribute_container_classes),
        number_of_classes + 1)

    with self.assertRaises(KeyError):
      manager.AttributeContainersManager.RegisterAttributeContainer(
          test_lib.TestAttributeContainer)

    manager.AttributeContainersManager.DeregisterAttributeContainer(
        test_lib.TestAttributeContainer)
    self.assertEqual(
        len(manager.AttributeContainersManager._attribute_container_classes),
        number_of_classes)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a parser for OXML files (i.e. MS Office 2007+)."""

import logging
import re
import struct
import zipfile

from xml.etree import ElementTree

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import interface
from plaso.parsers import manager


__author__ = 'David Nides (david.nides@gmail.com)'


class OpenXMLEventData(events.EventData):
  """OXML event data.

  Attributes:
    app_version (str): version of application that created document.
    author (str): name of author.
    creating_app (str): name of application that created document.
    doc_security (str): ???
    hyperlinks_changed (bool): True if hyperlinks have changed.
    i4 (str): ???
    last_saved_by (str): name of user that last saved the document.
    links_up_to_date (bool): True if the links are up to date.
    number_of_characters (int): number of characters without spaces in
        the document.
    number_of_characters_with_spaces (int): number of characters including
        spaces in the document.
    number_of_lines (int): number of lines in the document.
    number_of_pages (int): number of pages in the document.
    number_of_paragraphs (int): number of paragraphs in the document.
    number_of_words (int): number of words in the document.
    revision_number (int): revision number.
    scale_crop (bool): True if crop to scale is enabled.
    shared_doc (bool): True if document is shared.
    template (str): name of template ???
    total_time (str): ???
  """

  DATA_TYPE = u'metadata:openxml'

  def __init__(self):
    """Initializes event data."""
    super(OpenXMLEventData, self).__init__(data_type=self.DATA_TYPE)
    self.app_version = None
    self.author = None
    self.creating_app = None
    self.doc_security = None
    self.hyperlinks_changed = None
    self.i4 = None
    self.last_saved_by = None
    self.links_up_to_date = None
    self.number_of_characters = None
    self.number_of_characters_with_spaces = None
    self.number_of_lines = None
    self.number_of_pages = None
    self.number_of_paragraphs = None
    self.number_of_words = None
    self.revision_number = None
    self.scale_crop = None
    self.shared_doc = None
    self.template = None
    self.total_time = None


class OpenXMLParser(interface.FileObjectParser):
  """Parse metadata from OXML files."""

  NAME = u'openxml'
  DESCRIPTION = u'Parser for OpenXML (OXML) files.'

  _METAKEY_TRANSLATE = {
      u'creator': u'author',
      u'lastModifiedBy': u'last_saved_by',
      u'Total_Time': u'total_edit_time',
      u'Pages': u'number_of_pages',
      u'CharactersWithSpaces': u'number_of_characters_with_spaces',
      u'Paragraphs': u'number_of_paragraphs',
      u'Characters': u'number_of_characters',
      u'Lines': u'number_of_lines',
      u'revision': u'revision_number',
      u'Words': u'number_of_words',
      u'Application': u'creating_app',
      u'Shared_Doc': u'shared',
  }

  _FILES_REQUIRED = frozenset([
      u'[Content_Types].xml', u'_rels/.rels', u'docProps/core.xml'])

  def _FixString(self, key):
    """Convert CamelCase to lower_with_underscore."""
    # TODO: Add Unicode support.
    fix_key = re.sub(r'(.)([A-Z][a-z]+)', r'\1_\2', key)
    return re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', fix_key).lower()

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses an OXML file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    file_name = parser_mediator.GetDisplayName()

    if not zipfile.is_zipfile(file_object):
      raise errors.UnableToParseFile(
          u'[{0:s}] unable to parse file: {1:s} with error: {2:s}'.format(
              self.NAME, file_name, u'Not a Zip file.'))

    try:
      zip_container = zipfile.ZipFile(file_object, 'r')
    except (zipfile.BadZipfile, struct.error, zipfile.LargeZipFile):
      raise errors.UnableToParseFile(
          u'[{0:s}] unable to parse file: {1:s} with error: {2:s}'.format(
              self.NAME, file_name, u'Bad Zip file.'))

    zip_name_list = set(zip_container.namelist())

    if not self._FILES_REQUIRED.issubset(zip_name_list):
      raise errors.UnableToParseFile(
          u'[{0:s}] unable to parse file: {1:s} with error: {2:s}'.format(
              self.NAME, file_name, u'OXML element(s) missing.'))

    metadata = {}
    timestamps = {}

    try:
      rels_xml = zip_container.read(u'_rels/.rels')
    except zipfile.BadZipfile as exception:
      parser_mediator.ProduceExtractionError(
          u'Unable to parse file with error: {0:s}'.format(exception))
      return

    rels_root = ElementTree.fromstring(rels_xml)

    for properties in rels_root.iter():
      if u'properties' in repr(properties.get(u'Type')):
        try:
          xml = zip_container.read(properties.get(u'Target'))
          root = ElementTree.fromstring(xml)
        except (
            OverflowError, IndexError, KeyError, ValueError,
            zipfile.BadZipfile) as exception:
          logging.warning(
              u'[{0:s}] unable to read property with error: {1:s}.'.format(
                  self.NAME, exception))
          continue

        for element in root.iter():
          if element.text:
            _, _, tag = element.tag.partition(u'}')
            # Not including the 'lpstr' attribute because it is
            # very verbose.
            if tag == u'lpstr':
              continue

            if tag in (u'created', u'modified', u'lastPrinted'):
              timestamps[tag] = element.text
            else:
              tag_name = self._METAKEY_TRANSLATE.get(tag, self._FixString(tag))
              metadata[tag_name] = element.text

    event_data = OpenXMLEventData()
    event_data.app_version = metadata.get(u'app_version', None)
    event_data.author = metadata.get(u'author', None)
    event_data.creating_app = metadata.get(u'creating_app', None)
    event_data.doc_security = metadata.get(u'doc_security', None)
    event_data.hyperlinks_changed = metadata.get(u'hyperlinks_changed', None)
    event_data.i4 = metadata.get(u'i4', None)
    event_data.last_saved_by = metadata.get(u'last_saved_by', None)
    event_data.links_up_to_date = metadata.get(u'links_up_to_date', None)
    event_data.number_of_characters = metadata.get(
        u'number_of_characters', None)
    event_data.number_of_characters_with_spaces = metadata.get(
        u'number_of_characters_with_spaces', None)
    event_data.number_of_lines = metadata.get(u'number_of_lines', None)
    event_data.number_of_pages = metadata.get(u'number_of_pages', None)
    event_data.number_of_paragraphs = metadata.get(
        u'number_of_paragraphs', None)
    event_data.number_of_words = metadata.get(u'number_of_words', None)
    event_data.revision_number = metadata.get(u'revision_number', None)
    event_data.scale_crop = metadata.get(u'scale_crop', None)
    event_data.shared_doc = metadata.get(u'shared_doc', None)
    event_data.template = metadata.get(u'template', None)
    event_data.total_time = metadata.get(u'total_time', None)

    # Date and time strings are in ISO 8601 format with 1 second precision.
    # For example: 2012-11-07T23:29:00Z
    date_time = dfdatetime_time_elements.TimeElements()

    time_string = timestamps.get(u'created', None)
    if time_string:
      try:
        date_time.CopyFromStringISO8601(time_string)

        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_CREATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)
      except ValueError as exception:
        parser_mediator.ProduceExtractionError(
            u'unsupported created time: {0:s} with error: {1:s}.'.format(
                time_string, exception))

    time_string = timestamps.get(u'modified', None)
    if time_string:
      try:
        date_time.CopyFromStringISO8601(time_string)

        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)
      except ValueError as exception:
        parser_mediator.ProduceExtractionError(
            u'unsupported modified time: {0:s} with error: {1:s}.'.format(
                time_string, exception))

    time_string = timestamps.get(u'lastPrinted', None)
    if time_string:
      try:
        date_time.CopyFromStringISO8601(time_string)

        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_LAST_PRINTED)
        parser_mediator.ProduceEventWithEventData(event, event_data)
      except ValueError as exception:
        parser_mediator.ProduceExtractionError(
            u'unsupported last printed time: {0:s} with error: {1:s}.'.format(
                time_string, exception))


manager.ParsersManager.RegisterParser(OpenXMLParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Twitter on iOS 8+ plugin."""

import unittest

from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers.sqlite_plugins import twitter_ios

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class TwitterIOSTest(test_lib.SQLitePluginTestCase):
  """Tests for Twitter on iOS 8+ database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'twitter_ios.db'])
  def testProcess(self):
    """Test the Process function on a Twitter iOS file."""
    plugin = twitter_ios.TwitterIOSPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'twitter_ios.db'], plugin)

    # We should have 184 events in total.
    #  - 25 Contacts creation events.
    #  - 25 Contacts update events.
    #  - 67 Status creation events.
    #  - 67 Status update events.
    self.assertEqual(184, storage_writer.number_of_events)

    events = list(storage_writer.GetEvents())

    # Test the first contact creation event.
    event = events[0]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2007-04-22 14:42:37')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    self.assertEqual(event.screen_name, u'BBCBreaking')
    self.assertEqual(event.name, u'BBC Breaking News')
    self.assertEqual(event.location, u'London, UK')
    self.assertEqual(event.following, 0)
    self.assertEqual(event.followers_count, 19466932)
    self.assertEqual(event.following_count, 3)
    self.assertEqual(event.url, u'http://www.bbc.co.uk/news')

    expected_description = (
        u'Breaking news alerts and updates from the BBC. For news, features, '
        u'analysis follow @BBCWorld (international) or @BBCNews (UK). Latest '
        u'sport news @BBCSport.')

    self.assertEqual(event.description, expected_description)

    expected_profile_url = (
        u'https://pbs.twimg.com/profile_images/'
        u'460740982498013184/wIPwMwru_normal.png')

    self.assertEqual(event.profile_url, expected_profile_url)

    expected_message = (
        u'Screen name: BBCBreaking Profile picture URL: '
        u'https://pbs.twimg.com/profile_images/460740982498013184/'
        u'wIPwMwru_normal.png Name: BBC Breaking News Location: London, UK '
        u'Description: Breaking news alerts and updates from the BBC. For '
        u'news, features, analysis follow @BBCWorld (international) or '
        u'@BBCNews (UK). Latest sport news @BBCSport. URL: '
        u'http://www.bbc.co.uk/news Following: No Number of followers: '
        u'19466932 Number of following: 3')

    expected_short_message = (
        u'Screen name: BBCBreaking Description: Breaking news alerts and '
        u'updates from t...')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Test first contact modification event.
    event = events[1]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-12-02 15:35:44')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_UPDATE)

    self.assertEqual(event.screen_name, u'BBCBreaking')
    self.assertEqual(event.name, u'BBC Breaking News')
    self.assertEqual(event.location, u'London, UK')
    self.assertEqual(event.following, 0)
    self.assertEqual(event.followers_count, 19466932)
    self.assertEqual(event.following_count, 3)
    self.assertEqual(event.url, u'http://www.bbc.co.uk/news')

    expected_description = (
        u'Breaking news alerts and updates from the BBC. For news, features, '
        u'analysis follow @BBCWorld (international) or @BBCNews (UK). Latest '
        u'sport news @BBCSport.')

    self.assertEqual(event.description, expected_description)

    expected_profile_url = (
        u'https://pbs.twimg.com/profile_images/'
        u'460740982498013184/wIPwMwru_normal.png')

    self.assertEqual(event.profile_url, expected_profile_url)

    expected_message = (
        u'Screen name: BBCBreaking Profile picture URL: '
        u'https://pbs.twimg.com/profile_images/460740982498013184/'
        u'wIPwMwru_normal.png Name: BBC Breaking News Location: London, UK '
        u'Description: Breaking news alerts and updates from the BBC. For '
        u'news, features, analysis follow @BBCWorld (international) or '
        u'@BBCNews (UK). Latest sport news @BBCSport. URL: '
        u'http://www.bbc.co.uk/news Following: No Number of followers: '
        u'19466932 Number of following: 3')

    expected_short_message = (
        u'Screen name: BBCBreaking Description: Breaking news alerts and '
        u'updates from t...')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Test first status creation event.
    event = events[50]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-09-11 11:46:16')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    self.assertEqual(event.text, u'Never forget. http://t.co/L7bjWue1A2')
    self.assertEqual(event.user_id, 475222380)
    self.assertEqual(event.name, u'Heather Mahalik')
    self.assertEqual(event.retweet_count, 2)
    self.assertEqual(event.favorite_count, 3)
    self.assertEqual(event.favorited, 0)

    expected_message = (
        u'Name: Heather Mahalik User Id: 475222380 Message: Never forget. '
        u'http://t.co/L7bjWue1A2 Favorite: No Retweet Count: 2 Favorite '
        u'Count: 3')

    expected_short_message = (
        u'Name: Heather Mahalik Message: Never forget. http://t.co/L7bjWue1A2')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Test first status update event.
    event = events[51]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-12-02 15:39:37')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_UPDATE)

    self.assertEqual(event.text, u'Never forget. http://t.co/L7bjWue1A2')
    self.assertEqual(event.user_id, 475222380)
    self.assertEqual(event.name, u'Heather Mahalik')
    self.assertEqual(event.retweet_count, 2)
    self.assertEqual(event.favorite_count, 3)
    self.assertEqual(event.favorited, 0)

    expected_message = (
        u'Name: Heather Mahalik User Id: 475222380 Message: Never forget. '
        u'http://t.co/L7bjWue1A2 Favorite: No Retweet Count: 2 Favorite '
        u'Count: 3')

    expected_short_message = (
        u'Name: Heather Mahalik Message: Never forget. http://t.co/L7bjWue1A2')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The dpkg.log event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class DpkgFormatter(interface.ConditionalEventFormatter):
  """Formatter for a dpkg log file event."""

  DATA_TYPE = u'dpkg:line'

  FORMAT_STRING_SEPARATOR = u''

  FORMAT_STRING_PIECES = [
      u'{body}']

  SOURCE_LONG = u'dpkg log File'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(DpkgFormatter)



# -*- coding: utf-8 -*-
"""The OpenXML event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


__author__ = 'David Nides (david.nides@gmail.com)'


class OpenXMLParserFormatter(interface.ConditionalEventFormatter):
  """Formatter for an OXML event."""

  DATA_TYPE = u'metadata:openxml'

  FORMAT_STRING_PIECES = [
      u'Creating App: {creating_app}',
      u'App version: {app_version}',
      u'Title: {title}',
      u'Subject: {subject}',
      u'Last saved by: {last_saved_by}',
      u'Author: {author}',
      u'Total edit time (secs): {total_edit_time}',
      u'Keywords: {keywords}',
      u'Comments: {comments}',
      u'Revision number: {revision_number}',
      u'Template: {template}',
      u'Number of pages: {number_of_pages}',
      u'Number of words: {number_of_words}',
      u'Number of characters: {number_of_characters}',
      u'Number of characters with spaces: {number_of_characters_with_spaces}',
      u'Number of lines: {number_of_lines}',
      u'Company: {company}',
      u'Manager: {manager}',
      u'Shared: {shared}',
      u'Security: {security}',
      u'Hyperlinks changed: {hyperlinks_changed}',
      u'Links up to date: {links_up_to_date}',
      u'Scale crop: {scale_crop}',
      u'Digital signature: {dig_sig}',
      u'Slides: {slides}',
      u'Hidden slides: {hidden_slides}',
      u'Presentation format: {presentation_format}',
      u'MM clips: {mm_clips}',
      u'Notes: {notes}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Title: {title}',
      u'Subject: {subject}',
      u'Author: {author}']

  SOURCE_LONG = u'Open XML Metadata'
  SOURCE_SHORT = u'META'


manager.FormattersManager.RegisterFormatter(OpenXMLParserFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Opera history event formatters."""

import unittest

from plaso.formatters import opera

from tests.formatters import test_lib


class OperaGlobalHistoryFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Opera global history event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = opera.OperaGlobalHistoryFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = opera.OperaGlobalHistoryFormatter()

    expected_attribute_names = [
        u'url',
        u'title',
        u'description']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class OperaTypedHistoryFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Opera typed history event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = opera.OperaTypedHistoryFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = opera.OperaTypedHistoryFormatter()

    expected_attribute_names = [
        u'url',
        u'entry_selection']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*_ coding: utf-8 -*-
"""Tests for the Zsh extended_history parser."""
import unittest

from plaso.lib import timelib
from plaso.parsers import zsh_extended_history

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class ZshExtendedHistoryTest(test_lib.ParserTestCase):
  """Tests for the Zsh extended_history parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'zsh_extended_history.txt'])
  def testParse(self):
    """Tests for the Parse method."""
    parser = zsh_extended_history.ZshExtendedHistoryParser()
    storage_writer = self._ParseFile(
        [u'zsh_extended_history.txt'], parser)

    self.assertEqual(storage_writer.number_of_events, 4)

    events = list(storage_writer.GetEvents())

    event = events[0]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-03-12 08:26:50')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.elapsed_seconds, 0)
    self.assertEqual(event.command, u'cd plaso')

    event = events[2]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-03-26 11:54:53')
    expected_command = u'echo dfgdfg \\\\\n& touch /tmp/afile'
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.command, expected_command)

    event = events[3]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-03-26 11:54:57')
    self.assertEqual(event.timestamp, expected_timestamp)

  def testVerification(self):
    """Tests for the VerifyStructure method"""
    parser = zsh_extended_history.ZshExtendedHistoryParser()

    mediator = None
    valid_lines = u': 1457771210:0;cd plaso'
    self.assertTrue(parser.VerifyStructure(mediator, valid_lines))

    invalid_lines = u': 2016-03-26 11:54:53;0;cd plaso'
    self.assertFalse(parser.VerifyStructure(mediator, invalid_lines))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""ESEDB plugin related functions and classes for testing."""

import pyesedb

from plaso.containers import sessions
from plaso.parsers import esedb
from plaso.storage import fake_storage

from tests.parsers import test_lib


class ESEDBPluginTestCase(test_lib.ParserTestCase):
  """The unit test case for ESE database based plugins."""

  def _ParseESEDBFileWithPlugin(
      self, path_segments, plugin, knowledge_base_values=None):
    """Parses a file as an ESE database file and returns an event generator.

    Args:
      path_segments: a list of strings containinge the path segments inside
                     the test data directory.
      plugin: an ESE database plugin object (instance of ESEDBPlugin).
      knowledge_base_values: optional dictionary containing the knowledge base
                             values.

    Returns:
      A storage writer object (instance of FakeStorageWriter).
    """
    session = sessions.Session()
    storage_writer = fake_storage.FakeStorageWriter(session)
    storage_writer.Open()

    file_entry = self._GetTestFileEntry(path_segments)
    parser_mediator = self._CreateParserMediator(
        storage_writer, file_entry=file_entry,
        knowledge_base_values=knowledge_base_values)

    file_object = file_entry.GetFileObject()

    try:
      esedb_file = pyesedb.file()
      esedb_file.open_file_object(file_object)
      cache = esedb.ESEDBCache()
      plugin.Process(parser_mediator, cache=cache, database=esedb_file)
      esedb_file.close()

    finally:
      file_object.close()

    return storage_writer



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the analysis mediator."""

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.analysis import mediator
from plaso.containers import sessions
from plaso.storage import fake_storage

from tests.analysis import test_lib


class AnalysisMediatorTest(test_lib.AnalysisPluginTestCase):
  """Tests for the analysis mediator."""

  def testGetDisplayNameForPathSpec(self):
    """Tests the GetDisplayNameForPathSpec function."""
    session = sessions.Session()
    storage_writer = fake_storage.FakeStorageWriter(session)
    knowledge_base = self._SetUpKnowledgeBase()

    analysis_mediator = mediator.AnalysisMediator(
        storage_writer, knowledge_base)

    test_path = self._GetTestFilePath([u'syslog.gz'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    expected_display_name = u'OS:{0:s}'.format(test_path)
    display_name = analysis_mediator.GetDisplayNameForPathSpec(os_path_spec)
    self.assertEqual(display_name, expected_display_name)

  # TODO: add test for GetUsernameForPath.
  # TODO: add test for ProduceAnalysisReport.
  # TODO: add test for ProduceEventTag.

  def testSignalAbort(self):
    """Tests the SignalAbort function."""
    session = sessions.Session()
    storage_writer = fake_storage.FakeStorageWriter(session)
    knowledge_base = self._SetUpKnowledgeBase()

    analysis_mediator = mediator.AnalysisMediator(
        storage_writer, knowledge_base)

    analysis_mediator.SignalAbort()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows firewall log file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class WinFirewallFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows firewall log entry event."""

  DATA_TYPE = u'windows:firewall:log_entry'

  # TODO: Add more "elegant" formatting, as in transform ICMP code/type into
  # a more human readable format as well as translating the additional info
  # column (meaning may depend on action field).
  FORMAT_STRING_PIECES = [
      u'{action}',
      u'[',
      u'{protocol}',
      u'{path}',
      u']',
      u'From: {source_ip}',
      u':{source_port}',
      u'>',
      u'{dest_ip}',
      u':{dest_port}',
      u'Size (bytes): {size}',
      u'Flags [{flags}]',
      u'TCP Seq Number: {tcp_seq}',
      u'TCP ACK Number: {tcp_ack}',
      u'TCP Window Size (bytes): {tcp_win}',
      u'ICMP type: {icmp_type}',
      u'ICMP code: {icmp_code}',
      u'Additional info: {info}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{action}',
      u'[{protocol}]',
      u'{source_ip}',
      u': {source_port}',
      u'>',
      u'{dest_ip}',
      u': {dest_port}']

  SOURCE_LONG = u'Windows Firewall Log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(WinFirewallFormatter)
